@@ -1,9 +1,9 @@
 /*
   authc-hostbased.h
 
-  Authors: Sami Lehtinen <sjl@ssh.com>
+  Authors: Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-1999 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
                   
   Hostbased authentication, client-side.
@@ -20,7 +20,7 @@
 void ssh_client_auth_hostbased(SshAuthClientOperation op,
                                const char *user,
                                unsigned int packet_type,
-                               SshBuffer packet_in,
+                               SshBuffer *packet_in,
                                const unsigned char *session_id,
                                size_t session_id_len,
                                void **state_placeholder,
