@@ -2,14 +2,13 @@
 
 auths-passwd.c
 
-  Author: Tatu Ylonen <ylo@ssh.com>
+Author: Tatu Ylonen <ylo@cs.hut.fi>
 
-  Copyright (C) 1997 Tatu Ylonen <ylo@ssh.com>, Espoo, Finland
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
-  All rights reserved.
+Copyright (c) 1997 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
+                   All rights reserved
 
-  Password authentication, server-side.  This calls functions in 
-  machine-specific files to perform the actual authentication.
+Password authentication, server-side.  This calls functions in machine-specific
+files to perform the actual authentication.
 
 */
 
@@ -17,11 +16,10 @@ auths-passwd.c
 #include "sshencode.h"
 #include "sshauth.h"
 #include "sshmsgs.h"
+#include "auths-passwd.h"
 #include "sshuser.h"
 #include "sshserver.h"
 #include "sshconfig.h"
-#include "auths-common.h"
-#include "auths-passwd.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthPasswdServer"
 
@@ -30,7 +28,7 @@ auths-passwd.c
 
 SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
                                            const char *user,
-                                           SshBuffer packet,
+                                           SshBuffer *packet,
                                            const unsigned char *session_id,
                                            size_t session_id_len,
                                            void **state_placeholder,
@@ -43,90 +41,74 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
   Boolean change_request;
   char *password, *prompt;
   int disable_method = 0;
-  unsigned long pass_len = 0L;
-  
-  SSH_TRACE(6, ("password auth."));
-  
-  SSH_DEBUG(6, ("op = %d  user = %s", op, user));
+
+  SSH_DEBUG(6, ("auth_passwd op = %d  user = %s", op, user));
   
   switch (op)
     {
     case SSH_AUTH_SERVER_OP_START:
-      if (ssh_server_auth_check(&uc, user, config, server->common,
-                                SSH_AUTH_PASSWD))
+      if (uc == NULL)
         {
-          config->password_guesses--;
-          if (config->password_guesses <= 0)
+          uc = ssh_user_initialize(user, TRUE);
+          if (!uc)
             {
-              /* If this attempt is not succesful, disable this method. */
-              disable_method = 1;
-            }
-          goto password_bad;
+              /* If user context allocation failed, the user probably does not 
+                 exist. */
+              ssh_log_event(config->log_facility,
+                            SSH_LOG_WARNING,
+                            "User %s does not exist. "
+                            "(How did we get here?)", user);
+              return TRUE;
+            }       
         }
       *longtime_placeholder = (void *)uc;
       
       {
+        /* XXX it is possible to get rid of these. Modify
+           sshd2.c/auth_policy_proc*/
+        
         config->password_guesses--;
         if (config->password_guesses <= 0)
           {
             /* If this attempt is not succesful, disable this method. */
             disable_method = 1;
           }
-#ifndef SSHDIST_WINDOWS
-        if(ssh_user_uid(uc) == SSH_UID_ROOT &&
-           (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
-            config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
+        else if(ssh_user_uid(uc) == SSH_UID_ROOT &&
+                (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
+                 config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
           {
             /* XXX Add client addresses etc. */
-            SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
-                          "not permitted."));
             ssh_log_event(config->log_facility,
                           SSH_LOG_WARNING,
                           "root logins are not permitted.");
-
-            if (config->password_guesses > 0)
-              {
-                return SSH_AUTH_SERVER_REJECTED;
-              }
-            else
-              {
-                return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
-              }
+            SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
+                          "not permitted."));
+            return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
           }
-#endif /* SSHDIST_WINDOWS */
       }
       
       /* Parse the password authentication request. */
       if (ssh_decode_buffer(packet,
                             SSH_FORMAT_BOOLEAN, &change_request,
-                            SSH_FORMAT_UINT32_STR, &password, &pass_len,
+                            SSH_FORMAT_UINT32_STR, &password, NULL,
                             SSH_FORMAT_END) == 0)
         {
-          SSH_DEBUG(2, ("bad packet"));
+          SSH_DEBUG(2, ("ssh_server_auth_passwd: bad packet"));
           goto password_bad;
         }
 
-      if (!config->permit_empty_passwords && pass_len == 0L)
-        {
-          char *s = "login with empty passwords not permitted.";
-          ssh_log_event(config->log_facility,
-                        SSH_LOG_WARNING,
-                        "%s", s);
-          SSH_DEBUG(2, ("%s", s));
-          goto password_bad;
-        }
-      
       /* Password changing requests should only be received as continuation
          messages. */
       if (change_request)
         {
-          SSH_DEBUG(2 ,("changing password cannot start."));
+          SSH_DEBUG(2 ,("ssh_server_auth_passwd: changing password " \
+                        "cannot start."));
           goto password_bad;
         }
       
       /* Sanity check: do not pass excessively long passwords to system
          functions to avoid buffer overflows in operating system code. */
-      if (strlen(password) > 256)
+      if (strlen(password) > 64)
         {
           SSH_DEBUG(2, ("ssh_server_auth_passwd: password too long."));
           ssh_xfree(password);
@@ -159,7 +141,13 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
         }
 
       /* Try a local password (either normal or shadow). */
+#ifdef HAVE_SIA
+      if (ssh_user_validate_local_password(uc, 
+                                           password,
+                                           server->common->remote_host))
+#else /* HAVE_SIA */
       if (ssh_user_validate_local_password(uc, password))
+#endif /* HAVE_SIA */
         {
           ssh_log_event(config->log_facility,
                         SSH_LOG_NOTICE,
