@@ -2,15 +2,14 @@
 
 readpass.c
 
-  Author: Tatu Ylonen <ylo@ssh.com>
+Author: Tatu Ylonen <ylo@cs.hut.fi>
 
-  Copyright (C) 1995 Tatu Ylonen <ylo@ssh.com>, Espoo, Finland
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
-  All rights reserved.
+Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
+                   All rights reserved
 
-  Created: Mon Jul 10 22:08:59 1995 ylo
+Created: Mon Jul 10 22:08:59 1995 ylo
 
-  Functions for reading passphrases and passwords.
+Functions for reading passphrases and passwords.
 
 */
 
@@ -52,7 +51,7 @@ RETSIGTYPE ssh_rp_intr_handler(int sig)
    The passphrase if read from stdin if from_stdin is true (as is the
    case with ssh-keygen).  */
 
-char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
+char *ssh_read_passphrase(const char *prompt, int from_stdin)
 {
   char buf[1024], *cp;
   unsigned char quoted_prompt[512];
@@ -67,9 +66,7 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
   int i;
   
   if (from_stdin)
-    {
-      f = stdin;
-    }
+    f = stdin;
   else
     {
       /* Read the passphrase from /dev/tty to make it possible to ask it even 
@@ -182,66 +179,7 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
   tio.sg_flags &= ~(ECHO);
   ioctl(fileno(f), TIOCSETP, &tio);
 #endif
-  
-  {
-    /* Read the passphrase from the terminal. */
-    int read_bytes = 0, ret = 0;
-    while ((ret = read(fileno(f), &buf[read_bytes], 1)) != 0)
-      {
-        if (ret == -1)
-          {
-            if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR)
-              {
-                SSH_TRACE(2, ("got errno %d, system error %s",
-                              errno, strerror(errno)));
-                /* No more data at this time, or interrupted by system
-                   call.*/
-                continue;
-              }
-            else
-              {
-                /* Got some other error. */
-                SSH_TRACE(2, ("read() returned -1, with errno %d, "
-                              "system error message %s.",
-                              errno, strerror(errno)));
-                goto read_error;
-              }
-          }
 
-        read_bytes++;
-        if (buf[read_bytes - 1] == '\n' || buf[read_bytes - 1] == '\r' ||
-            read_bytes + 1>= sizeof(buf))
-          {
-            /* Got newline, error, or buf full. */
-            buf[read_bytes] = '\0';
-            break;            
-          }
-      }
-    if (ret == 0)
-      {
-        SSH_TRACE(2, ("Received EOF."));
-      read_error:
-        /* Got EOF, or error.  Just return NULL. */
-        /* Restore terminal modes. */
-#ifdef USING_TERMIOS
-        tcsetattr(fileno(f), TCSANOW, &saved_tio);
-#endif
-#ifdef USING_SGTTY
-        ioctl(fileno(f), TIOCSETP, &saved_tio);
-#endif
-        /* Restore the signal handler. */
-        signal(SIGINT, old_handler);
-        /* Print a newline (the prompt probably didn\'t have one). */
-        fprintf(stderr, "\r\n");
-        /* Close the file. */
-        if (f != stdin)
-          fclose(f);
-        return NULL;
-        
-      }
-  }
-
-#ifdef OLDXXX
   /* Read the passphrase from the terminal. */
   if (fgets(buf, sizeof(buf), f) == NULL)
     {
@@ -256,13 +194,12 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
       /* Restore the signal handler. */
       signal(SIGINT, old_handler);
       /* Print a newline (the prompt probably didn\'t have one). */
-      fprintf(stderr, "\r\n");
+      fprintf(stderr, "\n");
       /* Close the file. */
       if (f != stdin)
         fclose(f);
       return NULL;
     }
-#endif /* OLDXXX */
   /* Restore terminal modes. */
 #ifdef USING_TERMIOS
   tcsetattr(fileno(f), TCSANOW, &saved_tio);
@@ -275,15 +212,13 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
   /* Remove newline from the passphrase. */
   if (strchr(buf, '\n'))
     *strchr(buf, '\n') = 0;
-  if (strchr(buf, '\r'))
-    *strchr(buf, '\r') = 0;
   /* Allocate a copy of the passphrase. */
   cp = ssh_xstrdup(buf);
   /* Clear the buffer so we don\'t leave copies of the passphrase laying
      around. */
   memset(buf, 0, sizeof(buf));
   /* Print a newline since the prompt probably didn\'t have one. */
-  fprintf(stderr, "\r\n");
+  fprintf(stderr, "\n");
   /* Close the file. */
   if (f != stdin)
     fclose(f);
@@ -294,15 +229,13 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
 /* Reads a yes/no confirmation from /dev/tty.  Returns TRUE if "yes" is
    received.  Otherwise returns FALSE (also if EOF is encountered). */
 
-Boolean ssh_read_confirmation(const char *prompt, Boolean from_stdin)
+Boolean ssh_read_confirmation(const char *prompt)
 {
   char buf[1024], *p;
   FILE *f;
   
-  if (from_stdin)
-    {
-      f = stdin;
-    }
+  if (isatty(fileno(stdin)))
+    f = stdin;
   else
     {
       /* Read the passphrase from /dev/tty to make it possible to ask it even 
