@@ -3,20 +3,20 @@
 ssh-add.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.com>
-        Markku-Juhani Saarinen <mjos@ssh.com>
-        Timo J. Rinne <tri@ssh.com>
-        Sami Lehtinen <sjl@ssh.com>
+        Tatu Ylonen <ylo@ssh.fi>
+        Markku-Juhani Saarinen <mjos@ssh.fi>
+        Timo J. Rinne <tri@ssh.fi>
+        Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-1999 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-  Adds an identity to the authentication server, or removes an identity.
+Adds an identity to the authentication server, or removes an identity.
 
 */
 
 /*
- * $Id: ssh-add2.c,v 1.30 2000/05/05 14:12:42 tri Exp $
+ * $Id: ssh-add2.c,v 1.23 1999/05/04 02:14:41 kivinen Exp $
  * $Log: ssh-add2.c,v $
  * $EndLog$
  */
@@ -29,7 +29,7 @@ ssh-add.c
 #include "sshuser.h"
 #include "readpass.h"
 #include "sshuserfiles.h"
-#include "ssheloop.h"
+#include "sshunixeloop.h"
 #include "sshgetopt.h"
 #include "sshmiscstring.h"
 #ifdef WITH_PGP
@@ -38,9 +38,7 @@ ssh-add.c
 #include "sshpgp.h"
 #include "ssh2pgp.h"
 #include "ssh2pubkeyencode.h"
-#include "ssh1keydecode.h"
 #endif /* WITH_PGP */
-#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "SshAdd"
 
@@ -107,10 +105,10 @@ void agent_completion(SshAgentError result, void *context);
 
 void add_file(SshAgent agent, const char *filename)
 {
-  SshPrivateKey key = NULL;
+  SshPrivateKey key;
   char *saved_comment, *comment = NULL, *pass;
   int query_cnt;
-  unsigned char *certs = NULL;
+  unsigned char *certs;
   size_t certs_len;
   char privname[500], pubname[500];
   unsigned long magic;
@@ -178,7 +176,6 @@ void add_file(SshAgent agent, const char *filename)
       /* Read the public key blob. */
       magic = ssh2_key_blob_read(user, 
                                  pubname,
-                                 TRUE,
                                  &saved_comment,
                                  &certs, 
                                  &certs_len,
@@ -186,7 +183,6 @@ void add_file(SshAgent agent, const char *filename)
       if (magic != SSH_KEY_MAGIC_PUBLIC)
         {
           printf("Bad public key file %s\n", pubname);
-          ssh_xfree(certs);
           (*agent_completion)(SSH_AGENT_ERROR_OK, (void *)agent);
           return;
         }
@@ -242,26 +238,10 @@ void add_file(SshAgent agent, const char *filename)
               else
                 {
                   if (query_cnt <= 1)
-                    {
-                      if ((strcmp(privname, saved_comment) == 0) ||
-                          (((strlen(privname) + 4) == strlen(saved_comment)) &&
-                           (strncmp(privname, 
-                                    saved_comment, 
-                                    strlen(privname)) == 0)))
-                        {
-                          printf("Need passphrase for %s.\n", 
-                                 privname);
-                        }
-                      else
-                        {
-                          printf("Need passphrase for %s (%s).\n", 
-                                 privname, saved_comment);
-                        }
-                    }
+                    printf("Need passphrase for %s (%s).\n", 
+                           privname, saved_comment);
                   else
-                    {
-                      printf("Bad passphrase.\n");
-                    }
+                    printf("Bad passphrase.\n");
                   pass = ssh_read_passphrase("Enter passphrase: ", use_stdin);
                   if (pass == NULL || strcmp(pass, "") == 0)
                     {
@@ -322,7 +302,7 @@ void add_file(SshAgent agent, const char *filename)
     {
       unsigned char *blob, *public_blob;
       size_t blob_len, public_blob_len;
-      Boolean found = FALSE;
+      Boolean found;
       unsigned long id;
       char *endptr;
       SshPgpSecretKey pgp_key;
@@ -728,7 +708,7 @@ void usage()
 
 int main(int ac, char **av)
 {
-  int opt, i;
+  int opt, i, len;
   DIR *ssh2dir = NULL;
   char *ssh2dirname;
   Boolean dynamic_array = FALSE;
@@ -887,22 +867,26 @@ int main(int ac, char **av)
 
   /* Fetch default from ~/.ssh2/id_* (the first that we happen to get) */
 
-#define ID_PREFIX "id"
+#define ID_PREFIX "id_"
   
-  if (num_files == 0 && action != LIST && action != DELETE_ALL &&
-      action != LOCK && action != UNLOCK)
-    {
 #ifdef WITH_PGP
-      if (pgp_mode != PGP_KEY_NONE)
-        {
-          fprintf(stderr, "%s: Nothing to do!\n",  av0);
-          exit(EXIT_STATUS_ERROR);
-        }
+  if ((ac == 1) && (num_files == 0) && (pgp_mode != PGP_KEY_NONE))
+    {
+      fprintf(stderr, "%s: Nothing to do!",  av0);
+      exit(EXIT_STATUS_ERROR);
+    }
 #endif /* WITH_PGP */
-      ssh_dsprintf(&ssh2dirname, "%s/%s", ssh_user_dir(user), SSH_USER_DIR);
+
+  if (ac == 1 && num_files == 0)
+    {
+      /* len includes '/' and '\0'*/
+      len = strlen(ssh_user_dir(user)) + strlen(SSH_USER_DIR) + 2;
+      ssh2dirname = ssh_xcalloc(len, sizeof(char));
+      snprintf(ssh2dirname, len, "%s/%s", ssh_user_dir(user), SSH_USER_DIR);
+      
       ssh2dir = opendir(ssh2dirname);
 
-      if (!ssh2dir)
+      if (! ssh2dir)
         {
           fprintf(stderr, "%s: Can't open directory \"%s\"", av0, ssh2dirname);
           exit(EXIT_STATUS_ERROR);
@@ -910,21 +894,16 @@ int main(int ac, char **av)
           
       while ((cand = readdir(ssh2dir)) != NULL)
         {
-          if ((strlen(cand->d_name) > strlen(ID_PREFIX)) &&
-              (strncmp(cand->d_name, ID_PREFIX, strlen(ID_PREFIX)) == 0) &&
-              ((strlen(cand->d_name) < 4) ||
-               (strcmp(cand->d_name + strlen(cand->d_name) - 4,
-                       ".pub") != 0)) &&
-              ((((cand->d_name)[strlen(ID_PREFIX)]) == '_') ||
-               (((cand->d_name)[strlen(ID_PREFIX)]) == '-') ||
-               (((cand->d_name)[strlen(ID_PREFIX)]) == '.') ||
-               (((cand->d_name)[strlen(ID_PREFIX)]) == '(') ||
-               (((cand->d_name)[strlen(ID_PREFIX)]) == '[') ||
-               (((cand->d_name)[strlen(ID_PREFIX)]) == '<') ||
-               (((cand->d_name)[strlen(ID_PREFIX)]) == '>')))
+          if (strlen(cand->d_name) >= strlen(ID_PREFIX) &&
+              strncmp(cand->d_name, ID_PREFIX, strlen(ID_PREFIX)) == 0)
             {
               files = ssh_xcalloc(2, sizeof(char *));
-              ssh_dsprintf(&files[0], "%s/%s", ssh2dirname, cand->d_name);
+
+              /* len includes '/' and '\0'*/
+              len = strlen(ssh2dirname) + strlen(cand->d_name) + 2; 
+                                                                      
+              files[0] = ssh_xcalloc(len, sizeof(char));
+              snprintf(files[0],len, "%s/%s", ssh2dirname, cand->d_name);
               ssh_xfree(ssh2dirname);
               num_files++;
               dynamic_array = TRUE;
