@@ -1,17 +1,17 @@
 /*
 
-  ssh-agent.c
+ssh-agent.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.com>
-        Markku-Juhani Saarinen <mjos@ssh.com>
-        Timo J. Rinne <tri@ssh.com>
-        Sami Lehtinen <sjl@ssh.com>
+        Tatu Ylonen <ylo@ssh.fi>
+        Markku-Juhani Saarinen <mjos@ssh.fi>
+        Timo J. Rinne <tri@ssh.fi>
+        Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-  The ssh authentication agent.
+The ssh authentication agent.
 
 */
 
@@ -26,13 +26,9 @@
 #include "sshagentint.h"
 #include "sshuser.h"
 #include "sshuserfiles.h"
-#include "ssheloop.h"
+#include "sshunixeloop.h"
 #include "sshgetopt.h"
 #include "ssh2pubkeyencode.h"
-#ifdef WITH_SSH_AGENT1_COMPAT
-#include "ssh1encode.h"
-#include "ssh1keydecode.h"
-#endif /* WITH_SSH_AGENT1_COMPAT */
 #include "sshmp.h" /* was "gmp.h" */
 
 #define SSH_DEBUG_MODULE "SshAgent"
@@ -85,6 +81,7 @@ struct SshAgentImplRec {
   char *socket_name;
   char *socket_dir_name;
   char *lock_password;
+  SshRandomState random_state;
 #ifdef WITH_SSH_AGENT1_COMPAT
   /* Is `-1' flag on? */
   Boolean ssh1_compat;
@@ -183,7 +180,10 @@ Boolean ssh_agenti_invalid_key(SshAgentConnection conn,
 #ifdef WITH_SSH_AGENT1_COMPAT
 
 /* Make ssh1 style (length in two bytes) encoding of long integer */
-void ssh_agenti_ssh1_encode_mp(SshBuffer buffer, SshInt *n);
+void ssh_agenti_ssh1_encode_mp(SshBuffer *buffer, SshInt *n);
+
+/* Decode ssh1 style encoded long integer */
+Boolean ssh_agenti_ssh1_decode_mp(SshBuffer *buffer, SshInt *n);
 
 /* Reply to the list query from ssh-agent1 */
 void ssh_agenti_ssh1_list_keys(SshAgentConnection conn);
@@ -335,7 +335,7 @@ size_t ssh_split_comma_list(const char *str, char ***arr)
 Boolean ssh_agent_path_match(const char *rule, const char *path)
 {
   char **r, **p;
-  size_t nr = 0L, np = 0L, x = 0L, y = 0L;
+  size_t nr, np, x, y;
 
   /* Empty rule allows everything. */
   if ((rule == NULL) || (*rule == '\000'))
@@ -532,7 +532,7 @@ void ssh_agenti_list_keys(SshAgentConnection conn)
 {
   SshAgentKey key;
   SshUInt32 num_keys;
-  SshBufferStruct buffer;
+  SshBuffer buffer;
 
   if (conn->agent->lock_password)
     {
@@ -583,12 +583,9 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
   SshAgentKey key;
   const unsigned char *arg;
   size_t arg_len;
-  const unsigned char *session_id;
-  size_t session_id_len;
-  unsigned char *outputbuf = NULL, *tmp_buf = NULL;
-  size_t outputlen, tmp_buf_len;
+  unsigned char *outputbuf;
+  size_t outputlen;
   SshPrivateKey privkey;
-  SshHash hash;
 
   SSH_DEBUG(7, ("op=%s", op_name));
   key = ssh_agenti_find_key(conn->agent, public_blob, public_len, TRUE);
@@ -623,7 +620,7 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
       outputbuf = ssh_xmalloc(outputlen);
       if (ssh_private_key_sign_digest(privkey, arg, arg_len,
                                       outputbuf, outputlen,
-                                      &outputlen) !=
+                                      &outputlen, conn->agent->random_state) !=
           SSH_CRYPTO_OK)
         {
           SSH_DEBUG(6, ("sign failed"));
@@ -647,7 +644,7 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
       outputlen = ssh_private_key_max_signature_output_len(privkey);
       outputbuf = ssh_xmalloc(outputlen);
       if (ssh_private_key_sign(privkey, arg, arg_len, outputbuf, outputlen,
-                               &outputlen) !=
+                               &outputlen, conn->agent->random_state) !=
           SSH_CRYPTO_OK)
         {
           SSH_DEBUG(6, ("hash-and-sign failed"));
@@ -682,49 +679,6 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
                       SSH_FORMAT_UINT32_STR, outputbuf, outputlen,
                       SSH_FORMAT_END);
     }
-  else if (strcmp(op_name, "ssh1-challenge-response") == 0)
-    {
-      if (ssh_decode_array(data, len,
-                           SSH_FORMAT_UINT32_STR_NOCOPY, 
-                           &arg, &arg_len,
-                           SSH_FORMAT_UINT32_STR_NOCOPY, 
-                           &session_id, &session_id_len,
-                           SSH_FORMAT_END) != len)
-        {
-          SSH_DEBUG(3, ("ssh1-challenge-response: bad data"));
-          ssh_agenti_send_error(conn, SSH_AGENT_ERROR_UNSUPPORTED_OP);
-          goto cleanup_and_return;
-        }
-      outputlen = ssh_private_key_max_decrypt_output_len(privkey);
-      outputbuf = ssh_xmalloc(outputlen);
-      if (ssh_private_key_decrypt(privkey, arg, arg_len,
-                                  outputbuf, outputlen,
-                                  &outputlen) != SSH_CRYPTO_OK)
-        {
-          SSH_DEBUG(6, ("decrypt failed"));
-          ssh_agenti_send_error(conn, SSH_AGENT_ERROR_FAILURE);
-          goto cleanup_and_return;
-        }
-      if (ssh_hash_allocate("md5", &hash) != SSH_CRYPTO_OK)
-        {
-          SSH_DEBUG(6, ("hash allocate failed"));
-          ssh_agenti_send_error(conn, SSH_AGENT_ERROR_FAILURE);
-          goto cleanup_and_return;
-        }
-      ssh_hash_update(hash, 
-                      outputbuf, 
-                      (outputlen < 32) ? outputlen : 32);
-      ssh_hash_update(hash, 
-                      session_id, 
-                      session_id_len);
-      tmp_buf_len = ssh_hash_digest_length(hash);
-      tmp_buf = ssh_xmalloc(tmp_buf_len);
-      ssh_hash_final(hash, tmp_buf);
-      ssh_hash_free(hash);
-      ssh_agenti_send(conn, SSH_AGENT_OPERATION_COMPLETE,
-                      SSH_FORMAT_UINT32_STR, tmp_buf, tmp_buf_len,
-                      SSH_FORMAT_END);
-    }
   else
     {
       SSH_DEBUG(3, ("unknown op '%.50s'", op_name));
@@ -732,7 +686,6 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
     }
 
  cleanup_and_return:
-  ssh_xfree(outputbuf);
   return;
 }
 
@@ -749,7 +702,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
   char *description, *op_name, *forwarding_host, *password;
   SshUInt32 timeout_time;
   SshTime current_time;
-  SshBufferStruct buffer;
+  SshBuffer buffer;
   unsigned int byte;
   struct SshAgentKeyAttrsRec attrs;
 
@@ -1159,7 +1112,7 @@ void ssh_agenti_received_eof(void *context)
   if (!*connp)
     ssh_fatal("ssh_agenti_received_eof: connection %lx not found",
               (unsigned long)conn);
-  SSH_ASSERT(*connp == conn);
+  assert(*connp == conn);
   *connp = conn->next;
 
   /* Destroy and free the object.  This also closes the stream. */
@@ -1196,6 +1149,7 @@ SshAgentImpl ssh_agenti_create(char **path_return)
 
   agent = ssh_xcalloc(1, sizeof(*agent));
   agent->connections = NULL;
+  agent->random_state = NULL;
   agent->lock_password = NULL; /* Not locked */
   agent->listener = ssh_agenti_create_listener(getuid(), path_return,
                                                ssh_agenti_connection,
@@ -1598,7 +1552,7 @@ int main(int argc, char **argv)
   agent->socket_name = ssh_xstrdup(socket_name);
 
   /* Load the random seed file. */
-  ssh_randseed_open(user, NULL);
+  agent->random_state = ssh_randseed_open(user, NULL);
 
   /* Register a timeout to periodically check whether the parent has
      exited (which means we should exit too). */
@@ -1622,8 +1576,13 @@ int main(int argc, char **argv)
   /* Uninitialize the event loop. */
   ssh_event_loop_uninitialize();
 
-  /* Update the random seed file. */
-  ssh_randseed_update(user, NULL);
+  if (agent->random_state)
+    {
+      /* Update the random seed file. */
+      ssh_randseed_update(user, agent->random_state, NULL);
+      /* Free the random seed. */
+      ssh_random_free(agent->random_state);
+    }
 
   /* Free user database information about the current user. */
   ssh_user_free(user, FALSE);
@@ -1639,7 +1598,7 @@ int main(int argc, char **argv)
 }
 
 #ifdef WITH_SSH_AGENT1_COMPAT
-void ssh_agenti_ssh1_encode_mp(SshBuffer buffer, SshInt *n)
+void ssh_agenti_ssh1_encode_mp(SshBuffer *buffer, SshInt *n)
 {
   SshUInt32 len;
   unsigned char len_buf[2];
@@ -1661,12 +1620,30 @@ void ssh_agenti_ssh1_encode_mp(SshBuffer buffer, SshInt *n)
   return;
 }
 
+Boolean ssh_agenti_ssh1_decode_mp(SshBuffer *buffer, SshInt *n)
+{
+  SshUInt32 len;
+  unsigned char *len_buf;
+  size_t num_buf_len;
+
+  if (ssh_buffer_len(buffer) < 2)
+    return FALSE;
+  len_buf = ssh_buffer_ptr(buffer);
+  len = (len_buf[0] * 0x100) + len_buf[1];
+  num_buf_len = ((len + 7) >> 3) & 0xffff;
+  if (ssh_buffer_len(buffer) < (2 + num_buf_len))
+    return FALSE;
+  ssh_mp_set_buf(n, &(len_buf[2]), num_buf_len);
+  ssh_buffer_consume(buffer, num_buf_len + 2);
+  return TRUE;
+}
+
 void ssh_agenti_ssh1_list_keys(SshAgentConnection conn)
 {
   SshAgentKey key;
   SshAgentSsh1Key ssh1_key;
   unsigned long num_keys;
-  SshBufferStruct buffer;
+  SshBuffer buffer;
   char *key_name;
   SshPublicKey public_key;
   size_t bits;
@@ -1774,12 +1751,12 @@ void ssh_agenti_ssh1_list_keys(SshAgentConnection conn)
 }
 
 void ssh_agenti_ssh1_challenge(SshAgentConnection conn, 
-                               const unsigned char *data, 
-                               size_t len)
+                                   const unsigned char *data, 
+                                   size_t len)
 {
   SshInt e, n, challenge, output;
-  SshBufferStruct buffer;
-  SshBufferStruct reply;
+  SshBuffer buffer;
+  SshBuffer reply;
   SshUInt32 bits;
   SshPublicKey public_key = NULL;
   unsigned char *public_blob = NULL;
@@ -1813,9 +1790,9 @@ void ssh_agenti_ssh1_challenge(SshAgentConnection conn,
       SSH_DEBUG(3, ("failed to decode challenge key"));
       goto cleanup_reply_and_return;
     }
-  if (! (ssh1_decode_mp(&buffer, &e) &&
-         ssh1_decode_mp(&buffer, &n) &&
-         ssh1_decode_mp(&buffer, &challenge)))
+  if (! (ssh_agenti_ssh1_decode_mp(&buffer, &e) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &n) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &challenge)))
     {
       SSH_DEBUG(3, ("failed to decode challenge"));
       goto cleanup_reply_and_return;
@@ -1997,7 +1974,7 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
                              const unsigned char *data, 
                              size_t len)
 {
-  SshBufferStruct buffer;
+  SshBuffer buffer;
   SshInt n, e, d, u, p, q;
   SshUInt32 bits;
   SshPrivateKey private_key = NULL;
@@ -2023,12 +2000,12 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
     goto cleanup_reply_and_return;
 
   /* Get key numbers */
-  if (! (ssh1_decode_mp(&buffer, &n) &&
-         ssh1_decode_mp(&buffer, &e) &&
-         ssh1_decode_mp(&buffer, &d) &&
-         ssh1_decode_mp(&buffer, &u) &&
-         ssh1_decode_mp(&buffer, &p) &&
-         ssh1_decode_mp(&buffer, &q)))
+  if (! (ssh_agenti_ssh1_decode_mp(&buffer, &n) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &e) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &d) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &u) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &p) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &q)))
     goto cleanup_reply_and_return;
 
   /* Get key comment */
@@ -2044,7 +2021,7 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
    * be only visible for ssh-agent1.
    */
   /* Create the SSH private key from the extracted numbers */
-  if (ssh_private_key_generate(&private_key,
+  if (ssh_private_key_generate(conn->agent->random_state, &private_key,
                                SSH_CRYPTO_RSA,
                                SSH_PKF_SIZE, bits,
                                SSH_PKF_MODULO_N, &n,
@@ -2102,7 +2079,7 @@ void ssh_agenti_ssh1_remove_key(SshAgentConnection conn,
                                 const unsigned char *data, 
                                 size_t len)
 {
-  SshBufferStruct buffer;
+  SshBuffer buffer;
   SshInt n, e;
   SshUInt32 bits;
   SshPublicKey key = NULL;
@@ -2122,8 +2099,8 @@ void ssh_agenti_ssh1_remove_key(SshAgentConnection conn,
     goto cleanup_reply_and_return;
 
   /* Get public key numbers */
-  if (! (ssh1_decode_mp(&buffer, &e) &&
-         ssh1_decode_mp(&buffer, &n)))
+  if (! (ssh_agenti_ssh1_decode_mp(&buffer, &e) &&
+         ssh_agenti_ssh1_decode_mp(&buffer, &n)))
     goto cleanup_reply_and_return;
 
   success = ssh_agenti_ssh1key_delete(conn->agent, &n, &e);
