@@ -1,21 +1,18 @@
 /*
 
-  ssh-askpass.c
+ssh-askpass.c
 
-  Author: Antti Huima <huima@ssh.com>
-
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
-  All rights reserved.
+Copyright (c) 1997 SSH Communications Security, Finland
+                   All rights reserved
 
 */
 
-#define SSH_ALLOW_CPLUSPLUS_KEYWORDS
 #include "sshincludes.h"
 #include "sshgetopt.h"
 
 #ifdef time
 #undef time
-#endif
+#endif 
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -102,7 +99,7 @@ void fatal(char *fmt, ...)
 
 /* Open connection to the X display. */
 
-void open_display(void)
+void open_display()
 {
   char *display_name;
 
@@ -118,7 +115,7 @@ void open_display(void)
 
 /* Close the display after freeing data structures. */
 
-void close_display(void)
+void close_display()
 {
   XFreeModifiermap(modifiers);
   XUnloadFont(display, font);
@@ -128,7 +125,7 @@ void close_display(void)
 
 /* Create the graphics contexts. Color version. */
 
-void create_GCs(void)
+void create_GCs()
 {
   int i;
   XGCValues values;
@@ -147,7 +144,7 @@ void create_GCs(void)
 
 /* Create the graphics contexts. B/W version. */
 
-void create_GCs_bw(void)
+void create_GCs_bw()
 {
   int i;
   XGCValues values;
@@ -169,7 +166,7 @@ void create_GCs_bw(void)
 /* Try to allocate the colors we would like to use.
    If any of the allocations fails, revert to black'n'white mode. */
 
-void allocate_colors(void)
+void allocate_colors()
 {
   XColor xcolor, exact_color;
   int red, green, blue;
@@ -184,39 +181,39 @@ void allocate_colors(void)
   red = exact_color.red;
   blue = exact_color.blue;
   green = exact_color.green;
-
+  
   red += 16384;
   if (red > 65535) red = 65535;
   blue += 16384;
   if (blue > 65535) blue = 65535;
   green += 16384;
   if (green > 65535) green = 65535;
-
+  
   exact_color.red = red;
   exact_color.blue = blue;
   exact_color.green = green;
-
+  
   if(!XAllocColor(display, map, &exact_color))
     goto revert;
 
   color[hilight] = exact_color.pixel;
-
+  
   red -= 32768;
   if (red < 0) red = 0;
   blue -= 32768;
   if (blue < 0) blue = 0;
   green -= 32768;
   if (green < 0) green = 0;
-
+  
   exact_color.red = red;
   exact_color.blue = blue;
   exact_color.green = green;
-
+  
   if (!XAllocColor(display, map, &exact_color))
     goto revert;
 
   color[shadow] = exact_color.pixel;
-
+  
   if (!XAllocNamedColor(display, map, foreground_color_name,
                    &xcolor, &exact_color))
     goto revert;
@@ -261,7 +258,7 @@ void button_dimensions(char *string, int *width, int *height)
 
 /* Open windows (the dialog box and the cancel button). */
 
-void open_windows(void)
+void open_windows()
 {
   int i; int scrap;
   XSetWindowAttributes attributes;
@@ -300,7 +297,7 @@ void open_windows(void)
                                 CopyFromParent, CWCursor |
                                 CWBackingStore,
                                 &attributes);
-
+  
   leds = (w_width - relief * 2 - margin * 2 + (led_width - led_i_width)) /
     led_width;
   scrap = (w_width - relief * 2 - margin * 2 + (led_width - led_i_width)) %
@@ -309,7 +306,7 @@ void open_windows(void)
 
   led_state = ssh_xmalloc(sizeof(int) * leds);
   for (i = 0; i < leds; i++) led_state[i] = 0;
-
+  
   attributes.cursor = XCreateFontCursor(display, XC_top_left_arrow);
   XChangeWindowAttributes(display, cancel_button, CWCursor | CWBackingStore,
                           &attributes);
@@ -317,7 +314,7 @@ void open_windows(void)
 
 /* Map the windows. */
 
-void map_windows(void)
+void map_windows()
 {
   XMapRaised(display, main_window);
   XMapWindow(display, cancel_button);
@@ -325,7 +322,7 @@ void map_windows(void)
 
 /* Load the font wanted. */
 
-void get_font(void)
+void get_font()
 {
   font_struct = XLoadQueryFont(display, font_spec);
   if (!font_struct)
@@ -339,7 +336,7 @@ void get_font(void)
 
 /* Calculate correct geometry for the main window. */
 
-void compute_dimensions(void)
+void compute_dimensions()
 {
   Status status;
   Window root;
@@ -361,7 +358,7 @@ void compute_dimensions(void)
   w_height += relief * 2 + margin * 2;
   status = XGetGeometry(display, DefaultRootWindow(display),
                         &root, &dummy, &dummy, (unsigned int *)&d_width,
-                        (unsigned int *)&d_height, (unsigned int *)&dummy,
+                        (unsigned int *)&d_height, (unsigned int *)&dummy, 
                         (unsigned int *)&dummy);
   w_x = (d_width - w_width) / 2;
   w_y = (d_height - w_height) / 2;
@@ -379,7 +376,7 @@ void get_resource(char *name, char *class, char **where)
 
 /* Read and get some resources. */
 
-void read_resources(void)
+void read_resources()
 {
   Status err;
   unsigned char *prop_return;
@@ -407,7 +404,7 @@ void read_resources(void)
     {
       if (prop_return != NULL)
         {
-          database = XrmGetStringDatabase((char *) prop_return);
+          database = XrmGetStringDatabase(prop_return);
           XFree(prop_return);
         }
     }
@@ -472,7 +469,7 @@ void draw_led(int no, int on)
 {
   relief_box(leds_x + led_width * no, leds_y,
              leds_x + led_width * no + led_i_width,
-             leds_y + led_height, shadow, hilight, 2,
+             leds_y + led_height, shadow, hilight, 2, 
              on ? light_led : dark_led, main_window);
 }
 
@@ -499,7 +496,7 @@ void draw_button(char *string, Window window, int width, int height,
 
 /* Draw the dialog box and the cancel button. */
 
-void draw_it(void)
+void draw_it()
 {
   int y; int i;
   relief_box(0, 0, w_width - 1, w_height - 1, hilight, shadow, relief,
@@ -542,18 +539,18 @@ void button_release(XButtonEvent *event)
     }
 }
 
-void advance_leds(void)
+void advance_leds()
 {
   led_state[next_led] ^= 1;
   draw_led(next_led, led_state[next_led]);
   next_led = (next_led + 1) % leds;
 }
 
-void backward_leds(void)
+void backward_leds()
 {
   next_led--; if (next_led == -1) next_led += leds;
   led_state[next_led] ^= 1;
-  draw_led(next_led, led_state[next_led]);
+  draw_led(next_led, led_state[next_led]);  
 }
 
 /* Handle key press. */
@@ -589,7 +586,7 @@ void key_press(XKeyEvent *event)
           if (ppointer != phrase)
             {
               ppointer--;
-              backward_leds();
+              backward_leds();        
             }
           break;
           /* These cause the event loop to terminate. */
@@ -607,7 +604,7 @@ void key_press(XKeyEvent *event)
 }
 
 /* Read the modifiers mapping. */
-void check_keyboard(void)
+void check_keyboard()
 {
   modifiers = XGetModifierMapping(display);
 }
@@ -619,12 +616,12 @@ void timeout(int sig) /*ARGUSED*/
 }
 
 /* Event loop. */
-void event_loop(void)
+void event_loop()
 {
   int focused = 0;
   XEvent event;
   XSelectInput(display, main_window, ButtonPressMask |
-               VisibilityChangeMask | StructureNotifyMask |
+               VisibilityChangeMask | StructureNotifyMask |            
                ExposureMask | ButtonReleaseMask | KeyPressMask);
 
   /* Windows must be mapped not before XSelectInput, so that the
@@ -659,7 +656,7 @@ void event_loop(void)
               focused = 1;
             }
           }
-          break;
+          break;                     
         }
     }
   switch (exiting)
@@ -667,9 +664,9 @@ void event_loop(void)
     case 1:
       *ppointer = 0;
       printf("%s\n", phrase);
-      break;
+      break;     
     default:
-      break;
+      break;      
     }
 }
 
