@@ -3,12 +3,12 @@
   sshcommon.c
   
   Authors:
-        Tatu Ylonen <ylo@ssh.com>
-        Markku-Juhani Saarinen <mjos@ssh.com>
-        Timo J. Rinne <tri@ssh.com>
-        Sami Lehtinen <sjl@ssh.com>
+        Tatu Ylonen <ylo@ssh.fi>
+        Markku-Juhani Saarinen <mjos@ssh.fi>
+        Timo J. Rinne <tri@ssh.fi>
+        Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
   
   This file implements the core of the SSH2 channel protocols.  This interface
@@ -88,7 +88,6 @@ struct {
     ssh_channel_ssh1_agent_create, ssh_channel_ssh1_agent_destroy },
 #endif /* SSH_CHANNEL_SSH1_AGENT */
 
-#ifndef SSH_WIN_CLIENT
 #ifdef SSH_CHANNEL_TCPFWD
   /* Forwarded TCP/IP channels. */
   { "forwarded-tcpip", ssh_channel_ftcp_open_request, ssh_channel_ftcp_create,
@@ -99,14 +98,6 @@ struct {
   { "direct-tcpip", ssh_channel_dtcp_open_request, ssh_channel_dtcp_create,
     ssh_channel_dtcp_destroy },
 #endif /* SSH_CHANNEL_TCPFWD */
-#else /* !SSH_WIN_CLIENT */
-  /* Forwarded TCP/IP channels. */
-  { "forwarded-tcpip", ssh_channel_ftcp_open_request, ssh_channel_ftcp_create,
-    ssh_channel_ftcp_destroy },
-    /* Client doesn't accept forwards from the server. Hence the NULL below. */
-  { "direct-tcpip", NULL, ssh_channel_dtcp_create,
-    ssh_channel_dtcp_destroy },
-#endif /* !SSH_WIN_CLIENT */
 
   { NULL },
 };
@@ -124,45 +115,20 @@ void common_destroy_timeout(void *ctx)
 /* Processes a disconnect message received from the connection protocol.
    Currently, a DISCONNECT message is sent if the protocol terminates. */
 
-void ssh_common_disconnect(int reason, Boolean locally_generated,
-                           const char *msg, void *context)
+void ssh_common_disconnect(int reason, const char *msg, void *context)
 {
   SshCommon common = (SshCommon)context;
 
   SSH_DEBUG(2, ("DISCONNECT received: %s", msg));
 
-#if 0
-  /* When used with e.g. SCP, when the local scp closes the channel,
-     the connection protocol sends a CLOSE message to the server, which
-     responds immediately.  The server will then exit immediately, because all
-     channels have been closed.  Thus, the client will receive the server's CLOSE
-     response immediately followed by EOF from the connection.  The client will
-     get to process the EOF condition before it has destroyed its own
-     protocol modules, which may generate spurious Connection Lost messages
-     to the user.  This eliminates such messages when all channels
-     have been closed. */
-  if (common->client == TRUE && reason == SSH_DISCONNECT_CONNECTION_LOST &&
-      common->num_channels == 0)
-    return;
-#endif  
-  
   /* Log the disconnect in the system log. */
   ssh_log_event(common->config->log_facility, SSH_LOG_INFORMATIONAL,
-                "%s disconnected: %s",
-                locally_generated ? "Local" : "Remote host",msg);
+                "Remote host disconnected: %s", msg);
 
-  if (!common->authenticated)
-    {
-      /* Call the authentication_notify callback. */
-      if (common->authentication_notify)
-        (*common->authentication_notify)(common->user, FALSE,
-                                         common->context);
-    }
-  
   /* Call the disconnect function.  Note that it is always given.
      This will call ssh_common_destroy, destroying the SshCommon object
      and all channels. */
-  (*common->disconnect)(reason, locally_generated, msg, common->context);
+  (*common->disconnect)(reason, msg, common->context);
 }
 
 /* Processes a debug message received from the connection protocol. */
@@ -198,8 +164,7 @@ void ssh_common_special(SshCrossPacketType type, const unsigned char *data,
 
       /* XXX change to use data saved in HostBased authentication. */
       common->authenticated_client_host = ssh_xstrdup("XXX");
-      common->authenticated = TRUE;
-      
+
       /* In server, retrieve information about the authenticated user.  This
          is not used in the client. */
       if (!common->client)
@@ -212,10 +177,11 @@ void ssh_common_special(SshCrossPacketType type, const unsigned char *data,
         }
 
       /* Call the authenticated notify function if given. */
-      if (common->authentication_notify)
-        (*common->authentication_notify)(common->user, TRUE, common->context);
+      if (common->authenticated_notify)
+        (*common->authenticated_notify)(common->user, common->context);
 
       break;
+
     case SSH_CROSS_STARTUP:
     case SSH_CROSS_ALGORITHMS:
       /* Do nothing. */
@@ -258,7 +224,6 @@ void ssh_common_finalize(SshIpError error,
                                ssh_common_disconnect,
                                ssh_common_debug,
                                ssh_common_special,
-                               common->compat_flags,
                                (void *)common);
   SSH_DEBUG(5, ("connection protocol created"));
 }
@@ -269,20 +234,12 @@ void ssh_common_finalize(SshIpError error,
      `auth'         authentication protocol object
      `client'       TRUE if we are a client, FALSE if a server
      `config'       configuration data
-     `server_host_name'
-                    name of server host, or NULL in server
-     `disconnect'   function to call on disconnect (may be NULL)
-     `debug'        function to call on debug message (may be NULL)
-     `authenticated_notify'
-                    function to call when authenticated (may be NULL)
-     `destroy_notify'
-                    function to call when common object is being
-                    destroyed (may be NULL)
-     `compat_flags' compatibility flags, stored to the common
-                    struct. Should be a ssh_xmallocated object, which
-                    is freed by ssh_common_destroy. Can also be NULL,
-                    whence default values are used.
-     `context'      context to pass to ``destroy''
+     `random_state' initialized random state
+     `server_host_name' name of server host, or NULL in server
+     `disconnect'    function to call on disconnect (may be NULL)
+     `debug'         function to call on debug message (may be NULL)
+     `authenticated_notify' function to call when authenticated (may be NULL)
+     `context'       context to pass to ``destroy''
    The object should be destroyed from the ``disconnect'' callback or from
    a ``close_notify'' callback (see below).  */
 
@@ -290,12 +247,11 @@ SshCommon ssh_common_wrap(SshStream connection,
                           SshStream auth,
                           Boolean client,
                           SshConfig config,
+                          SshRandomState random_state,
                           const char *server_host_name,
                           SshConnDisconnectProc disconnect,
                           SshConnDebugProc debug,
-                          SshCommonAuthenticationNotify authentication_notify,
-                          SshCommonDestroyNotify destroy_notify,
-                          SshTransportCompat compat_flags,
+                          SshCommonAuthenticatedNotify authenticated_notify,
                           void *context)
 {
   SshCommon common;
@@ -314,19 +270,14 @@ SshCommon ssh_common_wrap(SshStream connection,
   common->num_channels = 0;
   common->disconnect = disconnect;
   common->debug = debug;
-  common->authentication_notify = authentication_notify;
-  common->destroy_notify = destroy_notify;
+  common->authenticated_notify = authenticated_notify;
   common->context = context;
+  common->random_state = random_state;
   common->server_host_name =
     server_host_name ? ssh_xstrdup(server_host_name) : NULL;
   common->being_destroyed = FALSE;
   common->auth = auth;
 
-  if (compat_flags)
-    common->compat_flags = compat_flags;
-  else
-    common->compat_flags = ssh_xcalloc(1, sizeof(*compat_flags));
-  
   common->last_login_time = 0;
   common->sizeof_last_login_from_host = 100;
   common->last_login_from_host =
@@ -346,11 +297,6 @@ SshCommon ssh_common_wrap(SshStream connection,
   if (!ssh_tcp_get_local_port(connection, common->local_port, 100))
     strcpy(common->local_port, "UNKNOWN");
 
-  SSH_DEBUG(0, ("local ip = %s, local port = %s", 
-                common->local_ip, common->local_port));
-  SSH_DEBUG(0, ("remote ip = %s, remote port = %s", 
-                common->remote_ip, common->remote_port));
-
   SSH_DEBUG(5, ("initializing channel types and requests"));
   
   /* Count the number of channel types and global requests. */
@@ -362,7 +308,7 @@ SshCommon ssh_common_wrap(SshStream connection,
       for (j = 0; ssh_channel_types[i].global_requests[j].name; j++)
         num_requests++;
     }
-  SSH_ASSERT(num_types == i);
+  assert(num_types == i);
 
   /* Allocate memory for channel types and global requests. */
   common->global_requests = ssh_xcalloc(num_requests + 1,
@@ -401,10 +347,10 @@ SshCommon ssh_common_wrap(SshStream connection,
                                (void *)common);
     }
   else
-    {
-      /* common->remote_host is allocated in ssh_common_finalize() */
+    {    
+      common->remote_host = ssh_xstrdup(common->remote_ip);
     }
-
+  
   if(client)
     ssh_common_finalize(SSH_IP_OK, common->remote_ip, common);
   
@@ -438,10 +384,6 @@ void ssh_common_destroy(SshCommon common)
         (*ssh_channel_types[i].type_destroy_proc)(common->type_contexts[i]);
       common->type_contexts[i] = NULL;
     }
-
-  /* Call the SshCommonDestroyNotify if given. */
-  if (common->destroy_notify)
-    (*common->destroy_notify)(common->context);
   
   ssh_xfree(common->type_contexts);
   common->type_contexts = NULL;
@@ -465,13 +407,6 @@ void ssh_common_destroy(SshCommon common)
   ssh_xfree(common->local_ip);
   ssh_xfree(common->local_port);
   ssh_xfree(common->authenticated_client_host);
-  ssh_xfree(common->compat_flags);
-  
-  if (common->remote_host)
-    ssh_xfree(common->remote_host);
-
-  if (common->last_login_from_host)
-    ssh_xfree(common->last_login_from_host);
 
   memset(common, 'F', sizeof(*common));
   ssh_xfree(common);
@@ -511,20 +446,13 @@ void ssh_common_destroy_channel(SshCommon common)
   common->num_channels--;
   SSH_DEBUG(1, ("num_channels now %d", common->num_channels));
 
-  /* XXX if all local or remote forwardings have failed, we should
-     exit. */
   /* If the common-object is already being destroyed, the set timeout would
      be accessing memory that has already been freed. That would not be
      legal. */
-    if (common->client && common->num_channels == 0 &&
-        (!common->no_session_channel ||
-         (common->config->one_shot_forwarding ||
-          !(common->config->local_forwards ||
-            common->config->remote_forwards))) &&
-        !common->being_destroyed)
+  if (common->num_channels == 0 && !common->no_session_channel && !common->being_destroyed)
     {
       /* Schedule the destruction of common struct from the bottom of the
-         event loop */
+       event loop */
       ssh_register_timeout(0L, 0L, common_destroy_timeout, (void *)common);
     }
 }
