@@ -3,12 +3,12 @@
   sshstdiofilter.h
 
   Authors:
-        Tatu Ylnen <ylo@ssh.com>
-        Markku-Juhani Saarinen <mjos@ssh.com>
-        Timo J. Rinne <tri@ssh.com>
-        Sami Lehtinen <sjl@ssh.com>
+        Tatu Ylnen <ylo@ssh.fi>
+        Markku-Juhani Saarinen <mjos@ssh.fi>
+        Timo J. Rinne <tri@ssh.fi>
+        Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
 */
@@ -23,14 +23,12 @@
 #ifndef SSHSTDIOFILTER_H
 #define SSHSTDIOFILTER_H
 
-#include "sshfilterstream.h"
-
-int ssh_stdio_output_filter(SshBuffer data,
+int ssh_stdio_output_filter(SshBuffer *data,
                             size_t offset,
                             Boolean eof_received,
                             void *context);
 
-int ssh_stdio_input_filter(SshBuffer data,
+int ssh_stdio_input_filter(SshBuffer *data,
                            size_t offset, 
                            Boolean eof_received,
                            void *context);
