@@ -2,17 +2,14 @@
 
   sshttyflags.h
   
-  Author: Sami Lehtinen <sjl@ssh.com>
-
-  Copyright (C) 1999-2000 SSH Communications Security Corp, Helsinki, Finland
-  All rights reserved.
+  Author: Sami Lehtinen <sjl@ssh.fi>
 
   Prototypes for public stty-mode manipulation functions.
   
 */
 
 /*
- * $Id: sshttyflags.h,v 1.5 2000/05/05 14:13:01 tri Exp $
+ * $Id: sshttyflags.h,v 1.2 1999/02/17 06:56:28 tri Exp $
  * $Log: sshttyflags.h,v $
  * $EndLog$
  */
@@ -22,6 +19,5 @@
 
 void ssh_decode_tty_flags(int fd, unsigned char *buf, size_t buf_len);
 void ssh_encode_tty_flags(int fd, unsigned char **buf, size_t *buf_len);
-void ssh1_encode_tty_flags(int fd, unsigned char **buf, size_t *buf_len);
 
 #endif /* SSHTTYFLAGS_H */
