@@ -16,16 +16,16 @@ cryptography and data security, including the following:
 
   Bruce Schneier: Applied Cryptography.  John Wiley & Sons, 1994.
 
-  Jennifer Seberry and Josed Pieprzyk: Cryptography: An Introduction to
+  Jennifer Seberry and Josed Pieprzyk: Cryptography: An Introduction to 
     Computer Security.  Prentice-Hall, 1989.
 
-  Man Young Rhee: Cryptography and Secure Data Communications.  McGraw-Hill,
+  Man Young Rhee: Cryptography and Secure Data Communications.  McGraw-Hill, 
     1994.
 
 */
 
 /*
- * $Id: des.c,v 1.27 2000/03/27 11:39:56 tmo Exp $
+ * $Id: des.c,v 1.24 1999/02/16 12:48:01 sjl Exp $
  * $Log: des.c,v $
  * $EndLog$
  */
@@ -380,7 +380,7 @@ const SshUInt32 SSH_CODE_SEGMENT ssh_des_SPtrans[8][64]={
         (a)^=((t)<<(n)))
 
 #if !defined(ASM_DES)
-
+     
 #define IP(l,r,t) \
         PERM_OP(r,l,t, 4,0x0f0f0f0f); \
         PERM_OP(l,r,t,16,0x0000ffff); \
@@ -412,12 +412,12 @@ const SshUInt32 SSH_CODE_SEGMENT ssh_des_SPtrans[8][64]={
 /* This part is based on code that used to be in ecb_enc.c. */
 /* Copyright (C) 1993 Eric Young - see README for more details */
 
-void ssh_des_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 *output, SshUInt32 *s,
+void ssh_des_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 *output, SshUInt32 *s, 
                  int for_encryption)
 {
   register SshUInt32 t,u;
   register int i;
-
+  
   IP(l,r,t);
 
   /* Things have been modified so that the initial rotate is
@@ -429,7 +429,7 @@ void ssh_des_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 *output, SshUInt32 *s,
   t=(r<<1)|(r>>31);
   r=(l<<1)|(l>>31);
   l=t;
-
+ 
   /* I don't know if it is worth the effort of loop unrolling the
    * inner loop */
   if (for_encryption)
@@ -476,7 +476,7 @@ void ssh_des_ede_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 output[2],
   t=(r<<1)|(r>>31);
   r=(l<<1)|(l>>31);
   l=t;
-
+ 
   /* I don't know if it is worth the effort of loop unrolling the
    * inner loop */
   if (for_encryption)
@@ -495,7 +495,7 @@ void ssh_des_ede_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 output[2],
         {
           D_ENCRYPT(l,r,i+0);
           D_ENCRYPT(r,l,i+2);
-        }
+        }      
     }
   else
     {
@@ -527,7 +527,7 @@ void ssh_des_ede_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 output[2],
 
 /* Prototypes for assembler code. */
 
-void ssh_des_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 *output, SshUInt32 *s,
+void ssh_des_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 *output, SshUInt32 *s, 
                  int for_encryption);
 
 void ssh_des_ede_encrypt(SshUInt32 l, SshUInt32 r, SshUInt32 output[2],
@@ -561,7 +561,7 @@ void ssh_des_set_key(const unsigned char *key, SshUInt32 *schedule)
   d = ((d & 0xff) << 16) | (d & 0xff00) |
     ((d >> 16) & 0xff) | ((c >> 4) & 0xf000000);
   c&=0x0fffffff;
-
+  
   shifts = 0x7efc;
   for (i=0; i < 16; i++)
     {
@@ -614,7 +614,7 @@ Boolean ssh_des_init(void *ptr, const unsigned char *key, size_t keylen,
 /* Table of weak keys that are checked for. This includes the usual
    weak and semi-weak keys. */
 #define SSH_DES_WEAK_KEYS  (4 + 6*2)
-static const unsigned char ssh_des_weak_keys[SSH_DES_WEAK_KEYS][8] =
+static unsigned char ssh_des_weak_keys[SSH_DES_WEAK_KEYS][8] =
 {
   /* The weak keys. */
   { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
@@ -625,16 +625,16 @@ static const unsigned char ssh_des_weak_keys[SSH_DES_WEAK_KEYS][8] =
   /* The semi-weak keys. */
   { 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe },
   { 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01, 0xfe, 0x01 },
-
+  
   { 0x1f, 0xe0, 0x1f, 0xe0, 0x0e, 0xf1, 0x0e, 0xf1 },
   { 0xe0, 0x1f, 0xe0, 0x1f, 0xf1, 0x0e, 0xf1, 0x0e },
-
+  
   { 0x01, 0xe0, 0x01, 0xe0, 0x01, 0xf1, 0x01, 0xf1 },
   { 0xe0, 0x01, 0xe0, 0x01, 0xf1, 0x01, 0xf1, 0x01 },
-
+  
   { 0x1f, 0xfe, 0x1f, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe },
   { 0xfe, 0x1f, 0xfe, 0x1f, 0xfe, 0x0e, 0xfe, 0x0e },
-
+  
   { 0x01, 0x1f, 0x01, 0x1f, 0x01, 0x0e, 0x01, 0x0e },
   { 0x1f, 0x01, 0x1f, 0x01, 0x0e, 0x01, 0x0e, 0x01 },
 
@@ -642,38 +642,29 @@ static const unsigned char ssh_des_weak_keys[SSH_DES_WEAK_KEYS][8] =
   { 0xfe, 0xe0, 0xfe, 0xe0, 0xfe, 0xf1, 0xfe, 0xf1 }
 };
 
-Boolean ssh_des_init_is_weak_key(const unsigned char *key)
+Boolean ssh_des_init_with_key_check(void *ptr,
+                                    const unsigned char *key, size_t keylen,
+                                    Boolean for_encryption)
 {
+  SshDESContext *ctx = (SshDESContext *)ptr;
   int i;
 
+  if (keylen != 8)
+    return FALSE;
+
   /* Do weak key checks. */
   for (i = 0; i < SSH_DES_WEAK_KEYS; i++)
     {
       int j, match;
-
+      
       for (j = 0, match = 0; j < 8; j++)
         if ((key[j] & 0xfe) == (ssh_des_weak_keys[i][j] & 0xfe))
           match++;
 
       /* Was a weak key? */
       if (match == 8)
-        return TRUE;
+        return FALSE;
     }
-  return FALSE;
-}
-
-
-Boolean ssh_des_init_with_key_check(void *ptr,
-                                    const unsigned char *key, size_t keylen,
-                                    Boolean for_encryption)
-{
-  SshDESContext *ctx = (SshDESContext *)ptr;
-
-  if (keylen != 8)
-    return FALSE;
-
-  if (ssh_des_init_is_weak_key(key))
-    return FALSE;
 
   /* Not a weak key continue. */
   ctx->for_encryption = for_encryption;
@@ -681,10 +672,10 @@ Boolean ssh_des_init_with_key_check(void *ptr,
 
   return TRUE;
 }
-
+                         
 void ssh_des_ecb(void *context, unsigned char *dest,
-                 const unsigned char *src, size_t len,
-                 unsigned char *iv)
+             const unsigned char *src, size_t len,
+             unsigned char *iv)
 {
   SshDESContext *ctx = (SshDESContext *)context;
   SshUInt32 output[2], l, r;
@@ -694,12 +685,12 @@ void ssh_des_ecb(void *context, unsigned char *dest,
     {
       l = SSH_GET_32BIT_LSB_FIRST(src);
       r = SSH_GET_32BIT_LSB_FIRST(src + 4);
-
+      
       ssh_des_encrypt(l, r, output, ctx->key_schedule, for_encryption);
-
+      
       SSH_PUT_32BIT_LSB_FIRST(dest, output[0]);
       SSH_PUT_32BIT_LSB_FIRST(dest + 4, output[1]);
-
+      
       src += 8;
       dest += 8;
       len -= 8;
@@ -707,8 +698,8 @@ void ssh_des_ecb(void *context, unsigned char *dest,
 }
 
 void ssh_des_cbc(void *context, unsigned char *dest,
-                 const unsigned char *src, size_t len,
-                 unsigned char *iv_arg)
+             const unsigned char *src, size_t len,
+             unsigned char *iv_arg)
 {
   SshDESContext *ctx = (SshDESContext *)context;
   SshUInt32 l, r, iv[2], temp[2];
@@ -716,7 +707,7 @@ void ssh_des_cbc(void *context, unsigned char *dest,
 
   iv[0] = SSH_GET_32BIT_LSB_FIRST(iv_arg);
   iv[1] = SSH_GET_32BIT_LSB_FIRST(iv_arg + 4);
-
+  
   if (for_encryption)
     {
       while (len)
@@ -745,13 +736,13 @@ void ssh_des_cbc(void *context, unsigned char *dest,
 
           temp[0] ^= iv[0];
           temp[1] ^= iv[1];
-
+          
           SSH_PUT_32BIT_LSB_FIRST(dest, temp[0]);
           SSH_PUT_32BIT_LSB_FIRST(dest + 4, temp[1]);
 
           iv[0] = l;
           iv[1] = r;
-
+          
           src += 8;
           dest += 8;
           len -= 8;
@@ -763,27 +754,27 @@ void ssh_des_cbc(void *context, unsigned char *dest,
 }
 
 void ssh_des_cfb(void *context, unsigned char *dest,
-                 const unsigned char *src, size_t len,
-                 unsigned char *iv)
+             const unsigned char *src, size_t len,
+             unsigned char *iv)
 {
   SshDESContext *ctx = (SshDESContext *)context;
   SshUInt32 l, r, temp[2];
 
   l = SSH_GET_32BIT_LSB_FIRST(iv);
   r = SSH_GET_32BIT_LSB_FIRST(iv + 4);
-
+  
   if (ctx->for_encryption)
     {
       while (len)
         {
-          ssh_des_encrypt(l, r, temp, ctx->key_schedule, TRUE);
-
+          ssh_des_encrypt(l, r, temp, ctx->key_schedule, TRUE); 
+          
           l = SSH_GET_32BIT_LSB_FIRST(src) ^ temp[0];
           r = SSH_GET_32BIT_LSB_FIRST(src + 4) ^ temp[1];
 
           temp[0] = l;
           temp[1] = r;
-
+          
           SSH_PUT_32BIT_LSB_FIRST(dest, temp[0]);
           SSH_PUT_32BIT_LSB_FIRST(dest + 4, temp[1]);
 
@@ -797,13 +788,13 @@ void ssh_des_cfb(void *context, unsigned char *dest,
       while (len)
         {
           ssh_des_encrypt(l, r, temp, ctx->key_schedule, TRUE);
-
+          
           l = SSH_GET_32BIT_LSB_FIRST(src);
           r = SSH_GET_32BIT_LSB_FIRST(src + 4);
 
           temp[0] ^= l;
           temp[1] ^= r;
-
+          
           SSH_PUT_32BIT_LSB_FIRST(dest, temp[0]);
           SSH_PUT_32BIT_LSB_FIRST(dest + 4, temp[1]);
 
@@ -818,28 +809,28 @@ void ssh_des_cfb(void *context, unsigned char *dest,
 }
 
 void ssh_des_ofb(void *context, unsigned char *dest,
-                 const unsigned char *src, size_t len,
-                 unsigned char *iv_arg)
+             const unsigned char *src, size_t len,
+             unsigned char *iv_arg)
 {
   SshDESContext *ctx = (SshDESContext *)context;
   SshUInt32 iv[2], l, r;
 
   iv[0] = SSH_GET_32BIT_LSB_FIRST(iv_arg);
   iv[1] = SSH_GET_32BIT_LSB_FIRST(iv_arg + 4);
-
+  
   while (len)
     {
       l = iv[0];
       r = iv[1];
-
+      
       ssh_des_encrypt(l, r, iv, ctx->key_schedule, TRUE);
-
+      
       l = SSH_GET_32BIT_LSB_FIRST(src) ^ iv[0];
       r = SSH_GET_32BIT_LSB_FIRST(src + 4) ^ iv[1];
-
+      
       SSH_PUT_32BIT_LSB_FIRST(dest, l);
       SSH_PUT_32BIT_LSB_FIRST(dest + 4, r);
-
+      
       src += 8;
       dest += 8;
       len -= 8;
@@ -860,50 +851,16 @@ Boolean ssh_des3_init(void *ptr, const unsigned char *key, size_t keylen,
                       Boolean for_encryption)
 {
   SshTripleDESContext *ctx = (SshTripleDESContext *)ptr;
-
+  
   ctx->for_encryption = for_encryption;
-
-  if (keylen != 24)
-    return FALSE;
-
+  
   ssh_des_set_key(key, &ctx->key_schedule[0]);
   ssh_des_set_key(&key[8], &ctx->key_schedule[1*32]);
   ssh_des_set_key(&key[16], &ctx->key_schedule[2*32]);
 
   return TRUE;
 }
-
-Boolean ssh_des3_init_with_key_check(void *ptr,
-                                     const unsigned char *key, size_t keylen,
-                                     Boolean for_encryption)
-{
-  SshTripleDESContext *ctx = (SshTripleDESContext *)ptr;
-
-  ctx->for_encryption = for_encryption;
-
-  if (keylen != 24)
-    return FALSE;
-
-  /* Check des weak keys. Is this really needed? Better put it here. */
-  if (ssh_des_init_is_weak_key(key))
-    return FALSE;
-  if (ssh_des_init_is_weak_key(key + 8))
-    return FALSE;
-  if (ssh_des_init_is_weak_key(key + 16))
-    return FALSE;
-
-  /* Check if K1 is same than K2, or K2 is same than K3. */
-  if (memcmp(key, key + 8, 8) == 0 ||
-      memcmp(key + 8, key + 16, 8) == 0)
-    return FALSE;
-
-  ssh_des_set_key(key, &ctx->key_schedule[0]);
-  ssh_des_set_key(&key[8], &ctx->key_schedule[1*32]);
-  ssh_des_set_key(&key[16], &ctx->key_schedule[2*32]);
-
-  return TRUE;
-}
-
+         
 void ssh_des3_ecb(void *context, unsigned char *dest,
               const unsigned char *src, size_t len,
               unsigned char *iv)
@@ -918,10 +875,10 @@ void ssh_des3_ecb(void *context, unsigned char *dest,
       r = SSH_GET_32BIT_LSB_FIRST(src + 4);
 
       ssh_des_ede_encrypt(l, r, output, ctx->key_schedule, for_encryption);
-
+          
       SSH_PUT_32BIT_LSB_FIRST(dest, output[0]);
       SSH_PUT_32BIT_LSB_FIRST(dest + 4, output[1]);
-
+      
       src += 8;
       dest += 8;
       len -= 8;
@@ -938,7 +895,7 @@ void ssh_des3_cbc(void *context, unsigned char *dest,
 
   iv[0] = SSH_GET_32BIT_LSB_FIRST(iv_arg);
   iv[1] = SSH_GET_32BIT_LSB_FIRST(iv_arg + 4);
-
+  
   if (for_encryption)
     {
       while (len)
@@ -967,13 +924,13 @@ void ssh_des3_cbc(void *context, unsigned char *dest,
 
           temp[0] ^= iv[0];
           temp[1] ^= iv[1];
-
+          
           SSH_PUT_32BIT_LSB_FIRST(dest, temp[0]);
           SSH_PUT_32BIT_LSB_FIRST(dest + 4, temp[1]);
 
           iv[0] = l;
           iv[1] = r;
-
+          
           src += 8;
           dest += 8;
           len -= 8;
@@ -993,13 +950,13 @@ void ssh_des3_cfb(void *context, unsigned char *dest,
 
   l = SSH_GET_32BIT_LSB_FIRST(iv);
   r = SSH_GET_32BIT_LSB_FIRST(iv + 4);
-
+  
   if (ctx->for_encryption)
     {
       while (len)
         {
-          ssh_des_ede_encrypt(l, r, temp, ctx->key_schedule, TRUE);
-
+          ssh_des_ede_encrypt(l, r, temp, ctx->key_schedule, TRUE); 
+          
           l = temp[0] ^= SSH_GET_32BIT_LSB_FIRST(src);
           r = temp[1] ^= SSH_GET_32BIT_LSB_FIRST(src + 4);
 
@@ -1012,7 +969,7 @@ void ssh_des3_cfb(void *context, unsigned char *dest,
         }
     }
   else
-    {
+    {      
       while (len)
         {
           ssh_des_ede_encrypt(l, r, temp, ctx->key_schedule, TRUE);
@@ -1022,7 +979,7 @@ void ssh_des3_cfb(void *context, unsigned char *dest,
 
           temp[0] ^= l;
           temp[1] ^= r;
-
+          
           SSH_PUT_32BIT_LSB_FIRST(dest, temp[0]);
           SSH_PUT_32BIT_LSB_FIRST(dest + 4, temp[1]);
 
@@ -1050,15 +1007,15 @@ void ssh_des3_ofb(void *context, unsigned char *dest,
     {
       l = iv[0];
       r = iv[1];
-
+      
       ssh_des_ede_encrypt(l, r, iv, ctx->key_schedule, TRUE);
-
+          
       l = SSH_GET_32BIT_LSB_FIRST(src) ^ iv[0];
       r = SSH_GET_32BIT_LSB_FIRST(src + 4) ^ iv[1];
-
+      
       SSH_PUT_32BIT_LSB_FIRST(dest, l);
       SSH_PUT_32BIT_LSB_FIRST(dest + 4, r);
-
+      
       src += 8;
       dest += 8;
       len -= 8;
