@@ -1,10 +1,10 @@
 /*
 
   libmonitor.c
-
+  
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Thu Jun 12 16:32:48 1997 [mkojo]
@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: libmonitor.c,v 1.3 2000/02/02 13:50:03 mtr Exp $
+ * $Id: libmonitor.c,v 1.2 1998/01/28 10:10:33 ylo Exp $
  * $Log: libmonitor.c,v $
  * $EndLog$
  */
@@ -32,19 +32,19 @@ void                    *ssh_crypto_progress_context = NULL;
 /* Interface from library. */
 
 void ssh_crypto_progress_monitor(SshCryptoProgressID id,
-                                 unsigned int time_value)
+				 unsigned int time_value)
 {
   if (ssh_crypto_progress_monitor_function != NULL)
     (*ssh_crypto_progress_monitor_function)(id, time_value,
-                                            ssh_crypto_progress_context);
+					    ssh_crypto_progress_context);
 }
 
 /* Interface from application. */
-
-void
+  
+DLLEXPORT void DLLCALLCONV
 ssh_crypto_library_register_progress_func(SshCryptoProgressMonitor
-                                          monitor_function,
-                                          void *progress_context)
+					  monitor_function,
+					  void *progress_context)
 {
   if (monitor_function == NULL)
     {
