@@ -23,7 +23,7 @@
   */
 
 /*
- * $Id: sshmp.h,v 1.21 2000/03/10 18:33:16 mkojo Exp $
+ * $Id: sshmp.h,v 1.17 1999/04/24 01:54:55 mkojo Exp $
  * $Log: sshmp.h,v $
  * $EndLog$
  */
@@ -325,33 +325,21 @@ typedef struct
 {
   /* Wether this context is initialized or not. */
   Boolean defined;
-
-  /* This number increases the speed, but also also the storage
-     amount. Basically speed increases "linearly", but storage requirements
-     increase exponentially. The linear increase, however, does not
-     hold in practice that well. */
-#define SSH_MP_POWM_BASE_K 6
-  
   /* A large table for computed values. */
   unsigned int table_size;
   unsigned int table_bits;
   SshIntModQ *table;
   /* The moduli under the table was computed. */
   SshIntModuli mod;
-  SshInt       order;
 } SshMpPowmBase;
 
 /* Initialize the base structure, performs the precomputation (which
    is very fast). */
-void ssh_mp_powm_with_base_init(SshInt *g, SshInt *m, SshInt *order,
+void ssh_mp_powm_with_base_init(SshInt *g, SshInt *m,
                                 SshMpPowmBase *base);
 /* Clears the base. */
 void ssh_mp_powm_with_base_clear(SshMpPowmBase *base);
 
-/* This is the general modular exponentiation with support for precomputed
-   base. */
-void ssh_mp_powm_with_base_dv(SshInt *ret, const SshInt *e,
-                              SshMpPowmBase *base);
 
 /* Following functions are the general modular exponentiation functions. */
 void ssh_mp_powm_naive(SshInt *op, const SshInt *g, const SshInt *e,
@@ -362,9 +350,11 @@ void ssh_mp_powm_naive_mont(SshInt *op, const SshInt *g, const SshInt *e,
                             const SshInt *m);
 void ssh_mp_powm_bsw_mont(SshInt *op, const SshInt *g, const SshInt *e,
                           const SshInt *m);
-void ssh_mp_powm_naive_mont_gg(SshInt *op, const SshInt *g1, const SshInt *e1,
-                               const SshInt *g2, const SshInt *e2,
-                               const SshInt *m);
+
+/* This is the general modular exponentiation with support for precomputed
+   base. */
+void ssh_mp_powm_with_base_bsw_mont(SshInt *ret, const SshInt *e,
+                                    SshMpPowmBase *base);
 
 /* Specialized routines for computing g^e (mod m), where g is very small. */
 void ssh_mp_powm_naive_ui(SshInt *op, SshWord g, const SshInt *e,
@@ -383,8 +373,7 @@ void ssh_mp_powm_naive_expui(SshInt *op, const SshInt *g, SshWord e,
 #define ssh_mp_powm_ui        ssh_mp_powm_naive_mont_ui
 #define ssh_mp_powm_base2     ssh_mp_powm_naive_mont_base2
 #define ssh_mp_powm_expui     ssh_mp_powm_naive_expui
-#define ssh_mp_powm_with_base ssh_mp_powm_with_base_dv
-#define ssh_mp_powm_gg        ssh_mp_powm_naive_mont_gg
+#define ssh_mp_powm_with_base ssh_mp_powm_with_base_bsw_mont
 
 /* Intermediate arithmetic routines. */
 
@@ -447,8 +436,6 @@ void ssh_mpm_clear(SshIntModQ *op);
 void ssh_mpm_set_mp(SshIntModQ *ret, const SshInt *op);
 /* Copy one value modulo m into another. I.e. ret = op. */
 void ssh_mpm_set(SshIntModQ *ret, const SshIntModQ *op);
-/* Copy unsigned int value to ret. */
-void ssh_mpm_set_ui(SshIntModQ *ret, SshWord u);
 
 /* Convert a value modulo m into SshInt. */
 void ssh_mp_set_mpm(SshInt *ret, const SshIntModQ *op);
@@ -457,8 +444,7 @@ void ssh_mp_set_m(SshInt *ret, const SshIntModuli *m);
 /* Comparison function. One should not rely on the fact that -1 and 1
    mean anything but that the inputs are different. 0 means always the
    the inputs are same. */
-int ssh_mpm_cmp(const SshIntModQ *op1, const SshIntModQ *op2);
-int ssh_mpm_cmp_ui(const SshIntModQ *op, SshWord u);
+int ssh_mpm_cmp(SshIntModQ *op1, SshIntModQ *op2);
   
 /* Basic arithmetic in modulo m representation. */
 
