@@ -8,7 +8,7 @@
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1998-2000 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Canonialize comma-separated cipher lists.
@@ -20,10 +20,7 @@
 
 /* list of ciphers in secsh draft */
 #define SSH_STD_CIPHERS \
-        "3des-cbc,blowfish-cbc,twofish-cbc,arcfour,idea-cbc,cast128-cbc,none"
-/* list of macs in secsh draft */
-#define SSH_STD_MACS \
-        "hmac-sha1,hmac-sha-96,hmac-md5,hmac-md5-96,none"
+        "3des-cbc,blowfish-cbc,arcfour,idea-cbc,cast128-cbc,twofish-cbc,none"
 
 /*
    True if list `list' contains item `item'.
@@ -35,21 +32,21 @@ Boolean ssh_cipher_list_contains(char *list, char *item);
    and names of the supported ones are always replaced with the
    `native' one.
 */
-char *ssh_cipher_list_canonicalize(char *str);
+char *ssh_cipher_list_canonialize(char *str);
 
 /*
    Canonialize pubkey algorithm names.  Unsupported algorithms are 
    excluded and names of the supported ones are always replaced 
    with the `native' one.
 */
-char *ssh_public_key_list_canonicalize(char *str);
+char *ssh_public_key_list_canonialize(char *str);
 
 /*
    Canonialize hash algorithm names.  Unsupported algorithms are 
    excluded and names of the supported ones are always replaced 
    with the `native' one.
 */
-char *ssh_hash_list_canonicalize(char *str);
+char *ssh_hash_list_canonialize(char *str);
 
 /* 
    Return a name list that contains items in list `original'
