@@ -8,10 +8,10 @@ sshconn.c
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-  SSH Connection Protocol.
+SSH Connection Protocol.
 
 */
 
@@ -46,7 +46,7 @@ typedef struct SshChannelRec
   /* If TRUE, the channel is still being created, and no other processing
      should be done for it. */
   Boolean ephemeral;
-
+  
   /* Data for extended channel types.  Normal data is read from index zero. */
   struct {
     /* The data stream, or NULL if there is no stream for this data type.
@@ -64,14 +64,14 @@ typedef struct SshChannelRec
     /* Flag indicating whether ``stream'' should be automatically
        closed when the channel is closed. */
     Boolean auto_close;
-
+    
     /* A read from the data stream has failed. */
     Boolean read_has_failed;
 
     /* EOF has been received from the channel. */
     Boolean eof_received;
-
-    /* Buffer for incoming data of this type.  This is allocated with
+    
+    /* SshBuffer for incoming data of this type.  This is allocated with
        ssh_xmalloc; its size is incoming_window_size.  This is used in a
        ring-buffer fashion. */
     unsigned char *buf;
@@ -82,9 +82,6 @@ typedef struct SshChannelRec
     /* Total number of bytes in the buffer. */
     size_t inbuf;
   } extended[MAX_EXTENDED_TYPES];
-
-  /* Sum of extended[*].inbuf over all extended types for this channel. */
-  size_t total_inbufs;
   
   /* The next type whose data we will transfer (0 = normal data). */
   unsigned int next_type;
@@ -99,11 +96,6 @@ typedef struct SshChannelRec
      when EOF is received from the channel stream. */
   Boolean close_on_eof;
 
-  /* TRUE if SSH_MSG_CHANNEL_CLOSE has been received for this channel.
-     This is basically true while output is draining; once it has drained,
-     close will be sent and the channel closed. */
-  Boolean close_received;
-  
   /* TRUE if SSH_MSG_CHANNEL_CLOSE has already been sent for this channel. */
   Boolean close_sent;
 
@@ -113,7 +105,7 @@ typedef struct SshChannelRec
   /* Callback to call when EOF received from the primary channel stream. */
   void (*eof_callback)(void *context);
   void *eof_context;
-
+  
   /* The number of bytes we can still send without receiving a window
      adjust message. */
   size_t outgoing_window_remaining;
@@ -130,7 +122,7 @@ typedef struct SshChannelRec
      This can be used to send smaller packets for interactive connections
      than for bulk data transfer. */
   size_t max_outgoing_packet_size;
-
+  
   /* Function to call when we receive a channel request. */
   SshConnChannelRequestProc request;
 
@@ -146,14 +138,13 @@ typedef struct SshChannelRec
 
   /* Context for ``open_callback''. */
   void *open_context;
-
+  
   /* Callback to be called when a reply is received for a channel request.
      This is NULL when we are not expecting a reply to a request. */
   SshConnSendChannelRequestCallback request_callback;
 
   /* Context argument to pass to the request callback. */
   void *request_context;
-
 } *SshChannel;
 
 struct SshConnRec
@@ -165,7 +156,7 @@ struct SshConnRec
   /* TRUE if ssh_conn_down_can_send has returned TRUE for ``down'', and we
      haven't yet received a can_send callback from it. */
   Boolean send_blocked;
-
+  
   /* Data for each channel.  This is indexed by the local channel number;
      each entry is either NULL (the channel does not exist) or a pointer
      to a SshChannel structure. */
@@ -182,7 +173,7 @@ struct SshConnRec
   /* The service name that we are going to accept.  We only accept this
      name. */
   char *service_name;
-
+  
   /* Array of global request name - function associations. */
   SshConnGlobalRequest *request_types;
 
@@ -210,9 +201,6 @@ struct SshConnRec
 
   /* This is set to TRUE when SSH_CROSS_AUTHENTICATED has been received. */
   Boolean authenticated;
-
-  /* Compatibility flags. */
-  SshTransportCompat compat_flags;
 };
 
 /* Allocates a new channel data structure, and allocates a local id for it.
@@ -237,7 +225,7 @@ SshChannel ssh_conn_channel_allocate(SshConn conn)
   /* Update the highest channel id if appropriate. */
   if (local_id > conn->highest_channel)
     conn->highest_channel = local_id;
-
+  
   /* Allocate and initialize the channel data structure.  Store it in the
      appropriate slot in the channels array. */
   channel = ssh_xcalloc(1, sizeof(*channel));
@@ -253,7 +241,7 @@ SshChannel ssh_conn_channel_allocate(SshConn conn)
 
   /* Store the new channel in the channels array. */
   conn->channels[local_id] = channel;
-
+  
   return channel;
 }
 
@@ -265,17 +253,17 @@ void ssh_conn_channel_free(SshConn conn, SshChannel channel)
   int i;
 
   /* We should never free a channel which is not allocated. */
-  SSH_ASSERT(conn->channels[channel->local_id] != NULL);
+  assert(conn->channels[channel->local_id] != NULL);
 
   /* Do this first to ensure that if the destroy callback destroys the
      connection protocol, we don't enter a recursive call to the same
      destroy function. */
   conn->channels[channel->local_id] = NULL;
-
+  
   /* Call the channel's destroy callback if set. */
   if (channel->destroy)
     (*channel->destroy)(channel->callback_context);
-
+  
   /* Close streams, and free any dynamically allocated data. */
   for (i = 0; i <= channel->highest_type; i++)
     {
@@ -320,7 +308,7 @@ Boolean ssh_conn_send_channel_data_type(SshConn conn, SshChannel channel,
           channel->extended[i].read_has_failed ||
           channel->extended[i].eof_received)
         return FALSE;
-
+      
       /* If cannot send, record that and return. */
       if (!ssh_cross_down_can_send(conn->down))
         {
@@ -362,7 +350,7 @@ Boolean ssh_conn_send_channel_data_type(SshConn conn, SshChannel channel,
           /* If an EOF callback has been registered, call it now. */
           if (channel->eof_callback != NULL)
             (*channel->eof_callback)(channel->eof_context);
-
+          
           /* Send an EOF or CLOSE message to the other side. */
           if (channel->close_on_eof)
             {
@@ -374,7 +362,6 @@ Boolean ssh_conn_send_channel_data_type(SshConn conn, SshChannel channel,
                                            channel->remote_id,
                                          SSH_FORMAT_END);
               channel->close_sent = TRUE;
-              channel->total_inbufs = 0;
             }
           else
             {
@@ -382,15 +369,15 @@ Boolean ssh_conn_send_channel_data_type(SshConn conn, SshChannel channel,
               ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
                                          SSH_FORMAT_CHAR,
                                          (unsigned int) SSH_MSG_CHANNEL_EOF,
-                                         SSH_FORMAT_UINT32, (SshUInt32)
+                                         SSH_FORMAT_UINT32, (SshUInt32) 
                                            channel->remote_id,
                                          SSH_FORMAT_END);
               channel->eof_sent = TRUE;
             }
-
+              
           return FALSE;
         }
-
+      
       /* Received some data from the stream.  Now wrap it into a
          packet and send to the other side. */
       if (i == 0)
@@ -436,7 +423,7 @@ Boolean ssh_conn_send_channel_data(SshConn conn, SshChannel channel)
   /* If the channel is still being created, return immediately. */
   if (channel->ephemeral || channel->close_sent || channel->eof_sent)
     return FALSE;
-
+  
   /* We iterate over the extended types in such a way that even if write to
      the central channel blocks and data is always available from a stream,
      fairness is always guaranteed.  */
@@ -464,7 +451,7 @@ Boolean ssh_conn_send_channel_data(SshConn conn, SshChannel channel)
 void ssh_conn_send_some_data(SshConn conn)
 {
   unsigned int i;
-
+  
   /* If write has failed, we'll eventually get a callback saying we can
      send again, and will retry then. */
   if (conn->send_blocked)
@@ -507,10 +494,10 @@ void ssh_conn_channel_check_adjust(SshConn conn, SshChannel channel)
   long adjust;
 
   ws = channel->incoming_window_size;
-
+  
   /* We only adjust after we have received at least half the window. */
   if (channel->incoming_window_received < ws / 2 ||
-      channel->eof_received || channel->close_sent || channel->close_received)
+      channel->eof_received || channel->close_sent)
     return;
 
   /* We only adjust if all extended types have enough buffer space
@@ -526,7 +513,7 @@ void ssh_conn_channel_check_adjust(SshConn conn, SshChannel channel)
 
   /* Compute the amount by which we adjust. */
   still_coming = ws - channel->incoming_window_received;
-  SSH_ASSERT(ws >= largest_inbuf + still_coming);
+  assert(ws >= largest_inbuf + still_coming);
   adjust = ws - largest_inbuf - still_coming;
 
   /* Send an adjust message to the other side. */
@@ -537,27 +524,7 @@ void ssh_conn_channel_check_adjust(SshConn conn, SshChannel channel)
                              SSH_FORMAT_UINT32, (SshUInt32) adjust,
                              SSH_FORMAT_END);
   channel->incoming_window_received -= adjust;
-}
-
-/* Send a close message for the channel back to the other end, and
-   destroy the channel.  This is called when a close message is
-   received from the other end, and all outgoing data has been
-   drained. */
-
-void ssh_conn_channel_send_close_and_destroy(SshConn conn, SshChannel channel)
-{
-  /* Send back a channel close message unless we have already sent one. */
-  if (!channel->close_sent)
-    ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
-                               SSH_FORMAT_CHAR,
-                               (unsigned int) SSH_MSG_CHANNEL_CLOSE,
-                               SSH_FORMAT_UINT32,
-                               (SshUInt32) channel->remote_id,
-                               SSH_FORMAT_END);
-      
-  /* Free the channel now. */
-  ssh_conn_channel_free(conn, channel);
-}
+}  
 
 /* Attempts to write data from channel buffers to the data streams.
    Sends a window adjust message if appropriate. */
@@ -568,10 +535,11 @@ void ssh_conn_channel_write(SshConn conn, SshChannel channel)
   int len, i;
   size_t ws;
 
-  /* If we have sent close to the channel, don't write to it. */
+  /* If we have sent close to the channel, don't write to it */
+  
   if (channel->close_sent)
     return;
-
+  
   did_something = FALSE;
   ws = channel->incoming_window_size;
 
@@ -606,49 +574,40 @@ void ssh_conn_channel_write(SshConn conn, SshChannel channel)
 
           if (len == 0)
             {
-              SSH_DEBUG(2, ("EOF received on write from channel 0x%lx, "
-                            "extended stream %d.", channel, i));
+              SSH_DEBUG(2, ("EOF received on write from channel 0x%lx, extended "\
+                            "stream %d.", channel, i));
 
               /* EOF received from one of the streams. */
               if (i != 0 || channel->close_sent)
                 break; /* We only process EOF from the main stream. */
 
               channel->extended[i].eof_received = TRUE;
-
+              
               /* If an EOF callback has been registered, call it now. */
               if (channel->eof_callback != NULL)
                 (*channel->eof_callback)(channel->eof_context);
-
-              /* Check if we were draining output. */
-              if (channel->close_received)
-                ssh_conn_channel_send_close_and_destroy(conn, channel);
-              else
-                {
-                  /* Send a close message for the channel. */
-                  ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
-                                             SSH_FORMAT_CHAR,
-                                             (unsigned int)
-                                               SSH_MSG_CHANNEL_CLOSE,
-                                             SSH_FORMAT_UINT32, (SshUInt32)
-                                             channel->remote_id,
-                                             SSH_FORMAT_END);
-                  channel->close_sent = TRUE;
-                  channel->total_inbufs -= channel->extended[i].inbuf;
-                  channel->extended[i].inbuf = 0;
-                  return;
-                }
+          
+              /* Send a close message for the channel. */
+              ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
+                                         SSH_FORMAT_CHAR,
+                                         (unsigned int) SSH_MSG_CHANNEL_CLOSE,
+                                         SSH_FORMAT_UINT32, (SshUInt32) 
+                                         channel->remote_id,
+                                         SSH_FORMAT_END);
+              channel->close_sent = TRUE;
+              
+              return;
             }
-
+          
           /* Mark that we actually did something. */
           did_something = TRUE;
 
           /* Update the ring buffer to consume the already written data. */
           channel->extended[i].start += len;
-          SSH_ASSERT(channel->extended[i].start <= ws);
+          assert(channel->extended[i].start <= ws);
           if (channel->extended[i].start == ws)
             channel->extended[i].start = 0;
           channel->extended[i].inbuf -= len;
-          channel->total_inbufs -= len;
           /* We loop again to process any remaining data in the buffer. */
         }
     }
@@ -656,13 +615,8 @@ void ssh_conn_channel_write(SshConn conn, SshChannel channel)
   /* If we did something, check whether we should adjust the window. */
   if (did_something)
     ssh_conn_channel_check_adjust(conn, channel);
-
-  /* If we were draining output, and all has been drained, send close
-     back and destroy now. */
-  if (channel->close_received && channel->total_inbufs == 0)
-    ssh_conn_channel_send_close_and_destroy(conn, channel);
 }
-
+      
 
 /* Stream callback for the channel streams.  This is called whenever
    we can send or receive data, or the stream is disconnected. */
@@ -689,7 +643,7 @@ void ssh_conn_channel_callback(SshStreamNotification op, void *context)
       ssh_debug("ssh_conn_channel_callback: got DISCONNECTED");
 #endif
       break;
-
+      
     default:
       ssh_fatal("ssh_conn_channel_callback: unexpected notification %d",
                 (int)op);
@@ -737,7 +691,7 @@ void ssh_conn_process_global_request(SshConn conn,
                                             data + bytes, len - bytes,
                                             conn->context))
           goto fail; /* Request failed. */
-
+        
         /* The request was successfully processed.  Send REQUEST_SUCCESS
            packet. */
         if (want_reply)
@@ -766,7 +720,7 @@ void ssh_conn_process_global_reply(SshConn conn, unsigned int packet_type,
                                    const unsigned char *data, size_t len)
 {
   SshConnSendGlobalRequestCallback cb;
-
+  
   if (len != 0)
     {
       ssh_cross_down_send_disconnect(conn->down, TRUE,
@@ -828,42 +782,19 @@ void ssh_conn_channel_open_completion(int result,
     {
       /* Open failed.  Free the channel data structures and send back
          a failure message. */
-      if (!(*conn->compat_flags->channel_open_failure_draft_incompatibility))
-        {
-          char *reason_string, *lang_tag;
-
-          reason_string = "Channel open failed.";
-          lang_tag = "en";
-          
-          ssh_cross_down_send_encode
-            (conn->down, SSH_CROSS_PACKET,
-             SSH_FORMAT_CHAR,
-             (unsigned int) SSH_MSG_CHANNEL_OPEN_FAILURE,
-             SSH_FORMAT_UINT32, (SshUInt32)
-             channel->remote_id,
-             SSH_FORMAT_UINT32, (SshUInt32) result,
-             SSH_FORMAT_UINT32_STR, reason_string, strlen(reason_string),
-             SSH_FORMAT_UINT32_STR, lang_tag, strlen(lang_tag),
-             SSH_FORMAT_END);
-        }
-      else
-        {
-          ssh_cross_down_send_encode
-            (conn->down, SSH_CROSS_PACKET,
-             SSH_FORMAT_CHAR,
-             (unsigned int) SSH_MSG_CHANNEL_OPEN_FAILURE,
-             SSH_FORMAT_UINT32, (SshUInt32)
-             channel->remote_id,
-             SSH_FORMAT_UINT32, (SshUInt32) result,
-             SSH_FORMAT_END);
-        }
-      
+      ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
+                                 SSH_FORMAT_CHAR,
+                                   (unsigned int) SSH_MSG_CHANNEL_OPEN_FAILURE,
+                                 SSH_FORMAT_UINT32, (SshUInt32)
+                                 channel->remote_id,
+                                 SSH_FORMAT_UINT32, (SshUInt32) result,
+                                 SSH_FORMAT_END);
       ssh_conn_channel_free(conn, channel);
       return;
     }
 
-  SSH_ASSERT(window_size < MAX_WINDOW_SIZE);
-
+  assert(window_size < MAX_WINDOW_SIZE);
+  
   /* Open was successful.  Finalize initializing the channel data structure. */
   channel->ephemeral = FALSE;
   channel->extended[0].stream = data_stream;
@@ -883,9 +814,9 @@ void ssh_conn_channel_open_completion(int result,
                              (unsigned int) SSH_MSG_CHANNEL_OPEN_CONFIRMATION,
                              SSH_FORMAT_UINT32, (SshUInt32) channel->remote_id,
                              SSH_FORMAT_UINT32, (SshUInt32) channel->local_id,
-                             SSH_FORMAT_UINT32, (SshUInt32)
+                             SSH_FORMAT_UINT32, (SshUInt32) 
                                channel->incoming_window_size,
-                             SSH_FORMAT_UINT32, (SshUInt32)
+                             SSH_FORMAT_UINT32, (SshUInt32) 
                                channel->max_outgoing_packet_size,
                              SSH_FORMAT_DATA, data, len,
                              SSH_FORMAT_END);
@@ -939,7 +870,7 @@ void ssh_conn_process_channel_open(SshConn conn,
   /* If no supported channel open types, fail. */
   if (conn->open_types == NULL)
     goto fail;
-
+  
   /* Go over all channel types, and call the appropriate callback if found.
      Otherwise, we fail the request. */
   for (i = 0; conn->open_types[i].name != NULL; i++)
@@ -965,7 +896,6 @@ void ssh_conn_process_channel_open(SshConn conn,
         channel->extended[0].buf = NULL;
         channel->extended[0].start = 0;
         channel->extended[0].inbuf = 0;
-        channel->total_inbufs = 0;
         channel->next_type = 0;
         channel->highest_type = 0;
         channel->outgoing_window_remaining = initial_window_size;
@@ -975,7 +905,7 @@ void ssh_conn_process_channel_open(SshConn conn,
         channel->request = NULL;
         channel->destroy = NULL;
         channel->callback_context = NULL;
-
+        
         /* Call the handler procedure.  It will call the completion
            procedure when done. */
         (*conn->open_types[i].proc)(channel_type, channel->local_id,
@@ -989,39 +919,15 @@ void ssh_conn_process_channel_open(SshConn conn,
       }
 
 fail:
-
-  /* The requested channel type is not supported.  Send a failure. */
-  if (!(*conn->compat_flags->channel_open_failure_draft_incompatibility))
-    {
-      char *reason_string, *lang_tag;
-
-      reason_string = "Requested channel type not supported.";
-      lang_tag = "en";
-          
-
-      ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
-                                 SSH_FORMAT_CHAR,
-                                 (unsigned int) SSH_MSG_CHANNEL_OPEN_FAILURE,
-                                 SSH_FORMAT_UINT32, remote_channel,
-                                 SSH_FORMAT_UINT32, (SshUInt32)
-                                 SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
-                                 SSH_FORMAT_UINT32_STR,
-                                 reason_string, strlen(reason_string),
-                                 SSH_FORMAT_UINT32_STR,
-                                 lang_tag, strlen(lang_tag),
-                                 SSH_FORMAT_END);
-    }
-  else
-    {
-      ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
-                                 SSH_FORMAT_CHAR,
-                                 (unsigned int) SSH_MSG_CHANNEL_OPEN_FAILURE,
-                                 SSH_FORMAT_UINT32, remote_channel,
-                                 SSH_FORMAT_UINT32, (SshUInt32)
-                                 SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
-                                 SSH_FORMAT_END);
-    }
   
+  /* The requested channel type is not supported.  Send a failure. */
+  ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
+                             SSH_FORMAT_CHAR,
+                             (unsigned int) SSH_MSG_CHANNEL_OPEN_FAILURE,
+                             SSH_FORMAT_UINT32, remote_channel,
+                             SSH_FORMAT_UINT32, (SshUInt32)
+                             SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
+                             SSH_FORMAT_END);
   ssh_xfree(channel_type);
 }
 
@@ -1102,26 +1008,12 @@ void ssh_conn_process_channel_open_failure(SshConn conn,
   SshInt32 local_id, result;
   SshChannel channel;
   SshConnSendChannelOpenCallback completion;
-  char *reason_string = NULL;
 
   /* Parse the packet. */
-  if (!(*conn->compat_flags->channel_open_failure_draft_incompatibility))
-    {
-      bytes = ssh_decode_array(data, len,
-                               SSH_FORMAT_UINT32, &local_id,
-                               SSH_FORMAT_UINT32, &result,
-                               SSH_FORMAT_UINT32_STR, &reason_string, NULL,
-                               SSH_FORMAT_UINT32_STR, NULL, NULL,
-                               SSH_FORMAT_END);
-    }
-  else
-    {    
-      bytes = ssh_decode_array(data, len,
-                               SSH_FORMAT_UINT32, &local_id,
-                               SSH_FORMAT_UINT32, &result,
-                               SSH_FORMAT_END);
-    }
-  
+  bytes = ssh_decode_array(data, len,
+                           SSH_FORMAT_UINT32, &local_id,
+                           SSH_FORMAT_UINT32, &result,
+                           SSH_FORMAT_END);
   if (bytes != len)
     {
       ssh_cross_down_send_disconnect(conn->down, TRUE,
@@ -1129,7 +1021,7 @@ void ssh_conn_process_channel_open_failure(SshConn conn,
                                      "Extra data at end of OPEN_FAILURE.");
       return;
     }
-  
+
   /* Check validity of the received channel number. */
   if (local_id < 0 || local_id > conn->highest_channel ||
       conn->channels[local_id] == NULL ||
@@ -1141,13 +1033,6 @@ void ssh_conn_process_channel_open_failure(SshConn conn,
       return;
     }
 
-  if (!(*conn->compat_flags->channel_open_failure_draft_incompatibility))
-    {
-      SSH_DEBUG(2, ("Opening channel %ld failed. Reason code: \"%s\"",
-                    local_id, reason_string));
-      ssh_xfree(reason_string);  
-    }
-
   channel = conn->channels[local_id];
 
   /* Call the user callback if non-NULL. */
@@ -1205,7 +1090,7 @@ void ssh_conn_process_channel_window_adjust(SshConn conn,
                                      "Bad window size in WINDOW_ADJUST.");
       return;
     }
-
+      
   /* Update the outgoing window. */
   channel->outgoing_window_remaining += bytes_to_add;
 
@@ -1223,7 +1108,7 @@ void ssh_conn_process_channel_data_common(SshConn conn,
 {
   SshChannel channel;
   size_t offset, ws;
-
+  
   /* Check validity of the received channel number. */
   if (local_id < 0 || local_id > conn->highest_channel ||
       conn->channels[local_id] == NULL ||
@@ -1239,12 +1124,6 @@ void ssh_conn_process_channel_data_common(SshConn conn,
   channel = conn->channels[local_id];
   ws = channel->incoming_window_size;
 
-  if (channel->close_sent)
-    {
-      SSH_DEBUG(1, ("Received data when close_sent - ignoring"));
-      return;
-    }
-  
   /* If ``type'' is invalid, force it to zero (normal data). */
   if (type < 0 || type > channel->highest_type ||
       channel->extended[type].stream == NULL)
@@ -1287,7 +1166,6 @@ void ssh_conn_process_channel_data_common(SshConn conn,
              len - (ws - offset));
     }
   channel->extended[type].inbuf += len;
-  channel->total_inbufs += len;
 
   /* Update the count of bytes received with this window.  Check if
      we should send a window adjust. */
@@ -1348,7 +1226,7 @@ void ssh_conn_process_channel_extended_data(SshConn conn,
                                      "Extra data at end of CHANNEL_DATA.");
       return;
     }
-
+  
   /* Do common processing. */
   ssh_conn_process_channel_data_common(conn, local_id, data_type, p, p_len);
 }
@@ -1395,7 +1273,7 @@ void ssh_conn_process_channel_eof(SshConn conn,
                                      "Postponed channel in CHANNEL_EOF.");
       return;
     }
-
+  
   /* Check for double-EOFs (we should only receive EOF once for a channel). */
   if (channel->eof_received)
     {
@@ -1407,7 +1285,7 @@ void ssh_conn_process_channel_eof(SshConn conn,
 
   /* Mark that we have received EOF for the channel. */
   channel->eof_received = TRUE;
-
+  
   /* Output EOF to every stream that does not have data in buffer (those
      that have data in buffer will send EOF once the buffer has drained). */
   for (i = 0; i <= channel->highest_type; i++)
@@ -1451,13 +1329,18 @@ void ssh_conn_process_channel_close(SshConn conn,
 
   channel = conn->channels[local_id];
 
-  /* Mark that we have received a close for this channel. */
-  channel->close_received = TRUE;
-
-  /* If there is no data to be drained, send close back and free the channel
-     immediately. */
-  if (channel->total_inbufs == 0)
-    ssh_conn_channel_send_close_and_destroy(conn, channel);
+  /* Send back a channel close message unless we have already sent one. */
+  if (!channel->close_sent)
+    ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
+                               SSH_FORMAT_CHAR,
+                               (unsigned int) SSH_MSG_CHANNEL_CLOSE,
+                               SSH_FORMAT_UINT32,
+                               (SshUInt32) channel->remote_id,
+                               SSH_FORMAT_END);
+      
+  /* Free the channel now. */
+  ssh_conn_channel_free(conn, channel);
+  
 }
 
 /* Processes a channel request message. */
@@ -1498,7 +1381,7 @@ void ssh_conn_process_channel_request(SshConn conn,
     }
 
   channel = conn->channels[local_id];
-
+  
   if (channel->request)
     result = (*channel->request)(type, data + bytes, len - bytes,
                                  channel->callback_context);
@@ -1549,7 +1432,7 @@ void ssh_conn_process_channel_reply(SshConn conn, unsigned int packet_type,
                                      "Bad local id in channel reply.");
       return;
     }
-
+  
   channel = conn->channels[local_id];
   cb = channel->request_callback;
   channel->request_callback = NULL;
@@ -1564,7 +1447,7 @@ void ssh_conn_process_channel_reply(SshConn conn, unsigned int packet_type,
                                      (int)packet_type, (int)local_id);
       return;
     }
-
+  
   /* Call the request callback. */
   if (packet_type == SSH_MSG_CHANNEL_SUCCESS)
     (*cb)(TRUE, data + bytes, len - bytes, channel->request_context);
@@ -1591,48 +1474,48 @@ void ssh_conn_process_packet(SshConn conn, unsigned int packet_type,
     case SSH_MSG_REQUEST_FAILURE:
       ssh_conn_process_global_reply(conn, packet_type, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_OPEN:
       ssh_conn_process_channel_open(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
       ssh_conn_process_channel_open_confirmation(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_OPEN_FAILURE:
       ssh_conn_process_channel_open_failure(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_WINDOW_ADJUST:
       ssh_conn_process_channel_window_adjust(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_DATA:
       ssh_conn_process_channel_data(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_EXTENDED_DATA:
       ssh_conn_process_channel_extended_data(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_EOF:
       ssh_conn_process_channel_eof(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_CLOSE:
       ssh_conn_process_channel_close(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_REQUEST:
       ssh_conn_process_channel_request(conn, data, len);
       break;
-
+      
     case SSH_MSG_CHANNEL_SUCCESS:
     case SSH_MSG_CHANNEL_FAILURE:
       ssh_conn_process_channel_reply(conn, packet_type, data, len);
       break;
-
+      
     default:
 #ifdef DEBUG
       ssh_debug("ssh_conn_process_packet: received unexpected packet %d",
@@ -1663,7 +1546,7 @@ void ssh_conn_received_packet(SshCrossPacketType type,
   unsigned int packet_type;
   size_t bytes;
   char *msg, *lang, *service;
-
+  
   switch (type)
     {
     case SSH_CROSS_PACKET:
@@ -1697,13 +1580,12 @@ void ssh_conn_received_packet(SshCrossPacketType type,
         ssh_fatal("ssh_conn_received_packet: bad DISCONNECT");
 
       if (conn->disconnect)
-        (*conn->disconnect)((int)reason_code, locally_generated,
-                            msg, conn->context);
+        (*conn->disconnect)((int)reason_code, msg, conn->context);
 
       ssh_xfree(msg);
       ssh_xfree(lang);
       break;
-
+      
     case SSH_CROSS_DEBUG:
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_BOOLEAN, &debug_type,
@@ -1728,16 +1610,7 @@ void ssh_conn_received_packet(SshCrossPacketType type,
       if (strcmp(service, SSH_CONNECTION_SERVICE) == 0)
         {
           /* Accept the service. */
-          SshBufferStruct buffer;
-          ssh_buffer_init(&buffer);
-          ssh_encode_buffer(&buffer,
-                            SSH_FORMAT_UINT32_STR, SSH_CONNECTION_SERVICE,
-                            strlen(SSH_CONNECTION_SERVICE),
-                            SSH_FORMAT_END);
-          ssh_cross_down_send(conn->down, SSH_CROSS_SERVICE_ACCEPT,
-                              ssh_buffer_ptr(&buffer),
-                              ssh_buffer_len(&buffer));
-          ssh_buffer_uninit(&buffer);
+          ssh_cross_down_send(conn->down, SSH_CROSS_SERVICE_ACCEPT, NULL, 0);
         }
       else
         {
@@ -1750,7 +1623,7 @@ void ssh_conn_received_packet(SshCrossPacketType type,
         }
       ssh_xfree(service);
       break;
-
+      
     case SSH_CROSS_AUTHENTICATED:
       /* Extract the service name from the packet. */
       if (ssh_decode_array(data, len,
@@ -1778,7 +1651,7 @@ void ssh_conn_received_packet(SshCrossPacketType type,
 
       ssh_xfree(service);
       break;
-
+      
       /* fall to next case */
     case SSH_CROSS_STARTUP:
       /* fall to next case */
@@ -1798,10 +1671,10 @@ void ssh_conn_received_packet(SshCrossPacketType type,
 void ssh_conn_received_eof(void *context)
 {
   SshConn conn = (SshConn)context;
-
+  
   if (conn->disconnect)
     (*conn->disconnect)(SSH_DISCONNECT_CONNECTION_LOST,
-                        TRUE, "Connection closed.", conn->context);
+                        "Connection closed.", conn->context);
 }
 
 /* Called when ssh_cross_down_can_send has returned FALSE and sending
@@ -1811,7 +1684,7 @@ void ssh_conn_received_eof(void *context)
 void ssh_conn_can_send(void *context)
 {
   SshConn conn = (SshConn)context;
-
+  
   /* Mark that sends are not blocked. */
   conn->send_blocked = FALSE;
 
@@ -1829,11 +1702,6 @@ void ssh_conn_can_send(void *context)
      `disconnect'          called if disconnect msg or EOF received, or NULL
      `debug'               called if debug msg received, or NULL
      `special'             called when a special packet is received, or NULL
-     `compat_flags'        compatibility flags. Is used to enable or disable
-                           compatibilty code with older versions. Has
-                           to be a valid struct, and it is not
-                           otherwise touched by the conn-stream,
-                           ie. has to freed by the caller, if need be.     
      `context'             argument to give to request/open functions.
    Any of the callbacks may be NULL to specify that it will be ignored. */
 
@@ -1844,15 +1712,14 @@ SshConn ssh_conn_wrap(SshStream auth_stream,
                       SshConnDisconnectProc disconnect,
                       SshConnDebugProc debug,
                       SshConnSpecialProc special,
-                      SshTransportCompat compat_flags,
                       void *context)
 {
   SshConn conn;
   int i;
 
-  SSH_DEBUG(2, ("Wrapping..."));
-
-  SSH_PRECOND(compat_flags);
+#ifdef DEBUG
+  ssh_debug("ssh_conn_wrap");
+#endif
   
   /* Allocate the context structure for the protocol. */
   conn = ssh_xcalloc(1, sizeof(*conn));
@@ -1880,8 +1747,7 @@ SshConn ssh_conn_wrap(SshStream auth_stream,
   conn->global_request_send_callback = NULL;
   conn->global_request_send_context = NULL;
   conn->authenticated = FALSE;
-  conn->compat_flags = compat_flags;
-  
+
   /* Enable receiving packets from the down stream. */
   ssh_cross_down_can_receive(conn->down, TRUE);
 
@@ -1901,7 +1767,7 @@ void ssh_conn_destroy(SshConn conn)
 #ifdef DEBUG
   ssh_debug("ssh_conn_destroy");
 #endif
-
+  
   /* Free all channels. */
   for (i = 0; i < MAX_OPEN_CHANNELS; i++)
     if (conn->channels[i] != NULL)
@@ -1914,7 +1780,7 @@ void ssh_conn_destroy(SshConn conn)
   /* Free the service name. */
   if (conn->service_name)
     ssh_xfree(conn->service_name);
-
+  
   /* Fill the context with a garbage value (to ease debugging) and free. */
   memset(conn, 'F', sizeof(*conn));
   ssh_xfree(conn);
@@ -1935,7 +1801,7 @@ void ssh_conn_send_disconnect(SshConn conn, int reason, const char *fmt, ...)
   va_end(va);
 }
 
-/* Sends a debug message to the stream.
+/* Sends a debug message to the stream. 
      `conn'      the connection protocol object
      `display'   whether `always_display' parameter will be set
      `fmt'       printf-style format string */
@@ -1950,7 +1816,7 @@ void ssh_conn_send_debug(SshConn conn, Boolean display, const char *fmt, ...)
       ssh_debug("ssh_conn_send_debug: cannot send - debug message ignored");
       return;
     }
-
+  
   /* Otherwise, send the message. */
   va_start(va, fmt);
   ssh_cross_down_send_debug_va(conn->down, display, fmt, va);
@@ -2021,8 +1887,8 @@ void ssh_conn_send_channel_open(SshConn conn,
 {
   SshChannel channel;
 
-  SSH_ASSERT(window_size < MAX_WINDOW_SIZE);
-
+  assert(window_size < MAX_WINDOW_SIZE);
+  
   /* Allocate a local channel number. */
   channel = ssh_conn_channel_allocate(conn);
   if (channel == NULL)
@@ -2046,7 +1912,7 @@ void ssh_conn_send_channel_open(SshConn conn,
   /* Save the open completion procedure. */
   channel->open_callback = completion;
   channel->open_context = completion_context;
-
+  
   /* Send an open request to the other side. */
   ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
                              SSH_FORMAT_CHAR,
@@ -2099,7 +1965,7 @@ void ssh_conn_send_channel_request(SshConn conn,
 
   /* We want a reply if ``cb'' is non-NULL. */
   want_reply = (cb != NULL);
-
+  
   /* Send the request packet. */
   ssh_cross_down_send_encode(conn->down, SSH_CROSS_PACKET,
                              SSH_FORMAT_CHAR,
@@ -2131,7 +1997,7 @@ void ssh_conn_channel_register_extended(SshConn conn,
                                         Boolean auto_close)
 {
   SshChannel channel;
-
+  
   /* Check that ``channel_id'' is valid. */
   if (channel_id < 0 || channel_id > conn->highest_channel ||
       conn->channels[channel_id] == NULL ||
@@ -2141,7 +2007,7 @@ void ssh_conn_channel_register_extended(SshConn conn,
 
   /* Save the callback. */
   channel = conn->channels[channel_id];
-
+  
   /* Check that the extended type value is sensible. */
   if (extended_type < 0 || extended_type >= MAX_EXTENDED_TYPES)
     ssh_fatal("ssh_conn_channel_register_extended: bad extended type %d",
@@ -2187,7 +2053,7 @@ void ssh_conn_channel_register_eof_callback(SshConn conn,
                                             void *context)
 {
   SshChannel channel;
-
+  
   /* Check that ``channel_id'' is valid. */
   if (channel_id < 0 || channel_id > conn->highest_channel ||
       conn->channels[channel_id] == NULL ||
@@ -2211,7 +2077,7 @@ void ssh_conn_channel_register_eof_callback(SshConn conn,
 void ssh_conn_channel_close(SshConn conn, int channel_id)
 {
   SshChannel channel;
-
+  
   /* Check that ``channel_id'' is valid. */
   if (channel_id < 0 || channel_id > conn->highest_channel ||
       conn->channels[channel_id] == NULL ||
@@ -2231,7 +2097,6 @@ void ssh_conn_channel_close(SshConn conn, int channel_id)
                              (unsigned int) SSH_MSG_CHANNEL_CLOSE,
                              SSH_FORMAT_UINT32, (SshUInt32) channel->remote_id,
                              SSH_FORMAT_END);
-  channel->total_inbufs = 0;
   channel->close_sent = TRUE;
 }
 
