@@ -8,15 +8,15 @@ sshcrdown.c
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-  Helper functions for the cross-layer protocol.  This file contains
-  functions to make it very easy to implement a cross layer stream (the
-  "Down" direction, which takes a downward going normal stream and gives a
-  nice packet based interface for the module using it).  Functions in this
-  module would be used in applications or protocol modules that sit above
-  a cross-layer based protocol module.
+Helper functions for the cross-layer protocol.  This file contains
+functions to make it very easy to implement a cross layer stream (the
+"Down" direction, which takes a downward going normal stream and gives a
+nice packet based interface for the module using it).  Functions in this
+module would be used in applications or protocol modules that sit above
+a cross-layer based protocol module.
 
 */
 
@@ -32,8 +32,6 @@ sshcrdown.c
 #include "sshencode.h"
 #include "sshcross.h"
 
-#define SSH_DEBUG_MODULE "SshProtoSshCrDown"
-
 struct SshCrossDownRec
 {
   /* The underlying stream going down.  This stream will be automatically
@@ -41,11 +39,11 @@ struct SshCrossDownRec
   SshStream stream;
 
   /* SshBuffer for incoming data (downwards). */
-  SshBufferStruct incoming;
+  SshBuffer incoming;
   Boolean incoming_eof;
-
+  
   /* SshBuffer for outgoing data (downwards). */
-  SshBufferStruct outgoing;
+  SshBuffer outgoing;
   Boolean outgoing_eof;
 
   /* Flag indicating that ssh_cross_down_can_send has returned FALSE, and
@@ -69,7 +67,7 @@ struct SshCrossDownRec
   /* Set to TRUE to request immediate destroy after returning from a
      callback. */
   Boolean destroy_requested;
-
+  
   /* Flag indicating that we have shortcircuited the stream.  If this is
      FALSE but shortcircuit_up_stream is non-NULL, we have a shortcircuit
      pending as soon as downward buffers have drained. */
@@ -78,7 +76,7 @@ struct SshCrossDownRec
   /* The stream to which we have shortcircuited.  NULL if not shortcircuited
      and no shortcircuit pending. */
   SshStream shortcircuit_up_stream;
-
+  
   /* Application callbacks. */
   SshCrossPacketProc received_packet;
   SshCrossEofProc received_eof;
@@ -127,7 +125,7 @@ Boolean ssh_cross_down_output(SshCrossDown down)
           ssh_buffer_clear(&down->outgoing);
           return TRUE;
         }
-
+      
       /* Consume written data. */
       ssh_buffer_consume(&down->outgoing, len);
 
@@ -138,7 +136,6 @@ Boolean ssh_cross_down_output(SshCrossDown down)
   /* All output has drained.  There is no more buffered data. */
   if (down->send_blocked)
     {
-      down->send_blocked = FALSE;
       down->cannot_destroy = TRUE;
       if (down->can_send)
         (*down->can_send)(down->context);
@@ -148,8 +145,9 @@ Boolean ssh_cross_down_output(SshCrossDown down)
           ssh_cross_down_destroy(down);
           return FALSE;
         }
+      down->send_blocked = FALSE;
     }
-
+  
   /* If we should send EOF after output has drained, do it now. */
   if (down->outgoing_eof)
     ssh_stream_output_eof(down->stream);
@@ -216,10 +214,10 @@ Boolean ssh_cross_down_input(SshCrossDown down)
           /* Packet header already in buffer. */
           ptr = ssh_buffer_ptr(&down->incoming);
           data_to_read = 4 + SSH_GET_32BIT(ptr);
-          SSH_ASSERT(data_to_read > data_read);
+          assert(data_to_read > data_read);
           ssh_buffer_append_space(&down->incoming, &ptr, data_to_read - data_read);
         }
-
+  
       /* Keep reading until entire packet read, or no more data available. */
       while (data_read < data_to_read)
         {
@@ -239,7 +237,7 @@ Boolean ssh_cross_down_input(SshCrossDown down)
               /* EOF received. */
               ssh_buffer_consume_end(&down->incoming, data_to_read - data_read);
               down->incoming_eof = TRUE;
-
+              
               /* Pass the EOF to the application callback. */
               down->cannot_destroy = TRUE;
               if (down->received_eof)
@@ -269,12 +267,12 @@ Boolean ssh_cross_down_input(SshCrossDown down)
         }
 
       /* An entire packet has been received. */
-      SSH_ASSERT(ssh_buffer_len(&down->incoming) == data_to_read);
-
+      assert(ssh_buffer_len(&down->incoming) == data_to_read);
+      
       /* Get packet type. */
       ptr = ssh_buffer_ptr(&down->incoming);
       type = (SshCrossPacketType)ptr[4];
-
+      
       /* Call the application callback if set. */
       down->cannot_destroy = TRUE;
       if (down->received_packet)
@@ -287,7 +285,7 @@ Boolean ssh_cross_down_input(SshCrossDown down)
           return FALSE;
         }
       ssh_buffer_clear(&down->incoming);
-
+      
       return_value = TRUE;
     }
   /*NOTREACHED*/
@@ -300,9 +298,9 @@ void ssh_cross_down_callback(SshStreamNotification op, void *context)
 {
   SshCrossDown down = (SshCrossDown)context;
   Boolean ret;
-
+  
   ret = FALSE;
-
+  
   /* Process the notification.  We loop between input and output
      operations until one returns FALSE (they return TRUE if the other
      operation should be performed). */
@@ -314,17 +312,17 @@ void ssh_cross_down_callback(SshStreamNotification op, void *context)
           ret = ssh_cross_down_output(down);
           op = SSH_STREAM_INPUT_AVAILABLE;
           break;
-
+          
         case SSH_STREAM_INPUT_AVAILABLE:
           ret = ssh_cross_down_input(down);
           op = SSH_STREAM_CAN_OUTPUT;
           break;
-
+          
         case SSH_STREAM_DISCONNECTED:
           ssh_debug("ssh_cross_down_callback: disconnected");
           ret = FALSE;
           break;
-
+          
         default:
           ssh_fatal("ssh_cross_down_callback: unknown op %d", (int)op);
         }
@@ -377,7 +375,7 @@ SshCrossDown ssh_cross_down_create(SshStream down_stream,
   /* Set callback for the downward stream.  Note that this will also cause
      can_send to be called from the output callback. */
   ssh_stream_set_callback(down->stream, ssh_cross_down_callback, (void *)down);
-
+  
   return down;
 }
 
@@ -458,7 +456,7 @@ Boolean ssh_cross_down_can_send(SshCrossDown down)
 
   return status;
 }
-
+  
 
 /* Encodes and sends a packet as specified for ssh_encode_cross_packet. */
 
