@@ -8,17 +8,17 @@ sshtrans.h
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-1998 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-  Created: Sat Sep  7 20:11:37 1996 ylo
+Created: Sat Sep  7 20:11:37 1996 ylo
 
-  SSH Transport Layer Protocol interface.
+SSH Transport Layer Protocol interface.
 
 */
 
 /*
- * $Id: sshtrans.h,v 1.20 2000/05/22 10:39:27 sjl Exp $
+ * $Id: sshtrans.h,v 1.11 1999/04/16 14:40:45 sjl Exp $
  * $Log: sshtrans.h,v $
  * $EndLog$
  */
@@ -75,7 +75,6 @@ typedef struct
   char *hash_algorithms;
   char *compressions_c_to_s;
   char *compressions_s_to_c;
-  int compression_level;
   char *ciphers_c_to_s;
   char *ciphers_s_to_c;
   char *macs_c_to_s;
@@ -114,8 +113,8 @@ typedef void (*SshKeyCheckCallback)(const char *server_name,
                            host, but without terminating newline.
       `context'            context argument that was supplied when the
                            callback was registered. */
-typedef Boolean (*SshVersionCallback)(const char *remote_version,
-                                      void *context);
+typedef void (*SshVersionCallback)(const char *remote_version,
+                                   void *context);
 
 /* Creates default transport protocol parameter structure.  This structure
    can be modified to choose different parameters. */
@@ -146,6 +145,7 @@ void ssh_transport_destroy_params(SshTransportParams params);
      `version_callback' is version check function, or NULL
      `version_context' is given as argument to `version_callback'. */
 SshStream ssh_transport_client_wrap(SshStream stream,
+                                    SshRandomState random_state,
                                     const char *version,
                                     const char *service,
                                     SshTransportParams params,
@@ -175,6 +175,7 @@ SshStream ssh_transport_client_wrap(SshStream stream,
      `version_callback' is version check function, or NULL
      `version_context' is given as argument to `version_callback'. */
 SshStream ssh_transport_server_wrap(SshStream stream,
+                                    SshRandomState random_state,
                                     const char *version,
                                     SshTransportParams params,
                                     SshPrivateKey private_host_key,
@@ -192,46 +193,18 @@ typedef struct {
   unsigned long uncompressed_outgoing_bytes;
   unsigned long incoming_packets;
   unsigned long outgoing_packets;
-
-  char *kex_name;
-  char *host_key_name;  /* Negotiated `default' public key algorithm */
-  char *host_key_names; /* List of common host key algorithms */
-
-  /* Client to server. */
-  char *cipher_c_to_s;
-  char *mac_c_to_s;
-  char *compression_c_to_s;
-
-  /* Server to client. */
-  char *cipher_s_to_c;
-  char *mac_s_to_c;
-  char *compression_s_to_c;  
 } SshTransportStatistics;
 
-/* Returns statistics information about the transport layer stream. */
+/* Returns statistics information about the transport layer object. */
 void ssh_transport_get_statistics(SshStream transport_stream,
                                   SshTransportStatistics *statistics_return);
 
-/* Returns statistics information about the transport layer
-   object. The ``tr_context'' can be grapped from a transport stream
-   with ssh_stream_get_context(). */
-void ssh_transport_get_statistics_tr(void *tr_context,
-                                     SshTransportStatistics *statistics_return);
-
 /* compat flags structure. These are pointers, because these can only
    fetched before the transport stream is wrapped. Their values change
    during the kexinit phase. */
 typedef struct 
 {
-  Boolean *publickey_draft_incompatibility;
-  Boolean *publickey_service_name_draft_incompatibility;
-  Boolean *x11_channel_open_draft_incompatibility;
-  Boolean *channel_open_failure_draft_incompatibility;
-  Boolean *publickey_pk_ok_draft_incompatibility;
-  Boolean *hostbased_service_name_draft_incompatibility;
-  Boolean *publickey_session_id_encoding_draft_incompatibility;
-  Boolean *malformed_signatures_draft_incompatibility;
-  Boolean *deprecated_disconnect_codes_draft_incompatibility;
+  Boolean *publickey_draft_incompatility;
 } *SshTransportCompat;
 
 /* Return application level compatibility flags. Note that this must
