@@ -23,6 +23,8 @@
 const char plaintext[] = 
   "huuhaa jeejee superjee niks naks supertampax";
 
+SshRandomState randseed;
+
 void simple_test(char *keytype, int keybits)
 {
   SshCryptoStatus code;
@@ -35,7 +37,7 @@ void simple_test(char *keytype, int keybits)
 
   /* generate a private key and a matching public key */
 
-  if (ssh_private_key_generate(&privkey, keytype,
+  if (ssh_private_key_generate(randseed, &privkey, keytype,
                                SSH_PKF_SIZE, keybits,
                                SSH_PKF_END) != SSH_CRYPTO_OK)
     ssh_fatal("simple_test: unable to generate %d - bit private key of type :"
@@ -65,9 +67,8 @@ void simple_test(char *keytype, int keybits)
   siglen = ssh_private_key_max_signature_output_len(privkey);
   signature = ssh_xmalloc(siglen);
   
-  code = ssh_private_key_sign(privkey, (unsigned char *)plaintext,
-                              strlen(plaintext),
-                              signature, siglen, &siglenr);
+  code = ssh_private_key_sign(privkey, plaintext, strlen(plaintext),
+                              signature, siglen, &siglenr, randseed);
   if (code != SSH_CRYPTO_OK)
     ssh_fatal("simple_test: ssh_private_key_sign() failed (%s).",
               ssh_crypto_status_message(code));
@@ -75,8 +76,7 @@ void simple_test(char *keytype, int keybits)
   /* ok, now verify the signature with our decoded public key */
 
   if (ssh_public_key_verify_signature(pubkey2, signature, siglenr,
-                                      (unsigned char *)plaintext,
-                                      strlen(plaintext) == FALSE))
+                                      plaintext, strlen(plaintext) == FALSE))
     ssh_fatal("simple_test: signature verification failed.");
               
   ssh_xfree(blob1);
@@ -93,11 +93,13 @@ int main(int argc, char **argv)
 {
   int i;
 
+  randseed = ssh_random_allocate();
   for (i = 0; i < T_PUBKEYENCODE_ITERATIONS; i++)
     {
       simple_test(SSH_CRYPTO_RSA, 64 * i + 512); 
       simple_test(SSH_CRYPTO_DSS, 64 * i + 512);
     }
+  ssh_random_free(randseed);
 
   return 0;
 }
