@@ -15,7 +15,9 @@ This is a simple implementation for generic unix platforms.
 
 #include "sshsessionincludes.h"
 #include "sshuser.h"
-#include "sshdsprintf.h"
+#ifdef HAVE_SIA
+#include "sshsia.h"
+#endif /* HAVE_SIA */
 
 #ifdef HAVE_SCO_ETC_SHADOW
 # include <sys/security.h>
@@ -26,7 +28,6 @@ This is a simple implementation for generic unix platforms.
 # endif
 #else /* HAVE_SCO_ETC_SHADOW */
 # ifdef HAVE_HPUX_TCB_AUTH
-#  undef ctime
 #  include <sys/types.h>
 #  include <hpsecurity.h>
 #  include <prot.h>
@@ -50,7 +51,6 @@ This is a simple implementation for generic unix platforms.
 #include <netdb.h>
 
 #ifdef HAVE_USERSEC_H
-#define SSH_ALLOW_CPLUSPLUS_KEYWORDS
 #include <usersec.h>
 #endif /* HAVE_USERSEC_H */
 
@@ -63,7 +63,6 @@ extern char *crypt(const char *key, const char *salt);
 struct SshUserRec
 {
   char *name;
-  char *group;
   char *dir;
   char *shell;
   char *correct_encrypted_passwd;
@@ -87,7 +86,7 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
   {
     char *expiration, current_time[100], normalized[100];
     int rlogin_permitted;
-    SshTime t;
+    ssh_time t;
     struct SshCalendarTimeRec tm[1];
     int account_is_locked;
     
@@ -338,7 +337,6 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
   uc = ssh_xcalloc(1, sizeof(*uc));
 
   uc->name = ssh_xstrdup(pw->pw_name);
-  uc->group = NULL;
   uc->dir = ssh_xstrdup(pw->pw_dir);
   uc->uid = pw->pw_uid;
   uc->gid = pw->pw_gid;
@@ -358,6 +356,12 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
       /* Save the encrypted password. */
       strncpy(correct_passwd, pw->pw_passwd, sizeof(correct_passwd));
 
+#ifdef HAVE_SIA
+      /* pr->pw_passwd may not be the real encrypted password (it won't be
+         under Enhanced Security), but we don't care because we never look at
+         uc->correct_encrypted_passwd.  We let my_sia_validate_user() do all
+         the work instead. */
+#else /* HAVE_SIA */
       /* If we have shadow passwords, lookup the real encrypted password from
          the shadow file, and replace the saved encrypted password with the
          real encrypted password. */
@@ -403,8 +407,8 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
         f = fopen("/etc/security/passwd", "r");
         if (f)
           {
-            snprintf(looking_for_user, sizeof(looking_for_user), 
-                     "%.190s:", ssh_user_name(uc));
+            /* XXX: user next line was server_user, is this OK? */
+            snprintf(looking_for_user, sizeof(looking_for_user), "%.190s:", user);
             while (fgets(line, sizeof(line), f))
               {
                 if (strchr(line, '\n'))
@@ -419,9 +423,7 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
                       {
                         for (cp = line; *cp == ' ' || *cp == '\t'; cp++)
                           ;
-                        if (strncmp(cp, 
-                                    "password = ", 
-                                    strlen("password = ")) == 0)
+                        if (strncmp(cp, "password = ", strlen("password = ")) == 0)
                           {
                             strncpy(correct_passwd, cp + strlen("password = "), 
                                     sizeof(correct_passwd));
@@ -437,6 +439,7 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
 #endif /* HAVE_ETC_SECURITY_PASSWD_ADJUNCT */
 #endif /* HAVE_ETC_SHADOW */
 #endif /* HAVE_SCO_ETC_SHADOW */
+#endif /* HAVE_SIA */
 
       uc->correct_encrypted_passwd = ssh_xstrdup(correct_passwd);
 
@@ -465,7 +468,6 @@ Boolean ssh_user_free(SshUser uc, Boolean undo)
   /* XXX undoing kerberos / secure rpc state. */
 
   ssh_xfree(uc->name);
-  ssh_xfree(uc->group);
   ssh_xfree(uc->dir);
   ssh_xfree(uc->shell);
   if (uc->correct_encrypted_passwd)
@@ -485,8 +487,14 @@ Boolean ssh_user_login_is_allowed(SshUser uc)
 
 /* Returns TRUE if login is allowed with the given local password. */
 
+#ifdef HAVE_SIA
+Boolean ssh_user_validate_local_password(SshUser uc,
+                                         const char *password,
+                                         const char *remote_host)
+#else /* HAVE_SIA */
 Boolean ssh_user_validate_local_password(SshUser uc,
                                          const char *password)
+#endif /* HAVE_SIA */
 {
   char *encrypted_password;
   const char *correct_passwd = uc->correct_encrypted_passwd;
@@ -503,8 +511,7 @@ Boolean ssh_user_validate_local_password(SshUser uc,
     svp = getsvc();
     if (svp == NULL)
       {
-        SSH_DEBUG(2, ("getsvc() failed in ultrix code in auth_passwd",
-                      strerror(errno)));
+        error("getsvc() failed in ultrix code in auth_passwd");
         return FALSE;
       }
     if ((svp->svcauth.seclevel == SEC_UPGRADE &&
@@ -514,6 +521,36 @@ Boolean ssh_user_validate_local_password(SshUser uc,
   }
 #endif /* HAVE_ULTRIX_SHADOW_PASSWORDS */
 
+#ifdef HAVE_SIA
+  {
+    int argc;
+    char **argv;
+
+    /* Passing a collection routine to my_sia_validate_user() here would
+       be useless and could be harmful.
+
+       It would be useless because at this point, stdin/stdout/stderr
+       are all redirected to /dev/null.
+
+       It would be harmful if the collection routine tried to print
+       anything to stdout, say, a warning that the password is incorrect.
+       At this point, nothing has been printed to stdout yet and it's
+       redirected to /dev/null.  Printing to stdout now would initialize
+       it as fully buffered (not line buffered) since /dev/null isn't a
+       tty.  That would cause problems later on in the child, which gets
+       a copy of the stdout structure.  When the child printed /etc/motd
+       and the mail notice, the user wouldn't see them because they'd
+       be stuck in the stdout buffer. */
+
+    get_sia_args(&argc, &argv);
+    if (my_sia_validate_user(NULL, argc, argv,
+                             (char *)remote_host, uc->name,
+                             NULL, 0, NULL, (char *)password) == SIASUCCESS)
+      return TRUE;
+    else
+      return FALSE;
+  }
+#endif /* HAVE_SIA */
 
   /* Encrypt the candidate password using the proper salt. */
 #if defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH)
@@ -696,7 +733,7 @@ Boolean ssh_user_become(SshUser uc)
 #ifdef HAVE_SETLOGIN
   /* Set login name in the kernel.  Warning: setsid() must be called before
      this. */
-  if (setlogin((char *)ssh_user_name(uc)) < 0)
+  if (setlogin(ssh_user_name(uc)) < 0)
     SSH_DEBUG(2, ("setlogin failed: %.100s", strerror(errno)));
 #endif /* HAVE_SETLOGIN */
 
@@ -711,19 +748,21 @@ Boolean ssh_user_become(SshUser uc)
   /* Close any extra file descriptors.  Note that there may still be
      descriptors left by system functions.  They will be closed later. */
   endpwent();
-
-#if 0
   endhostent();
-#endif
-  
+
   /* Close any extra open file descriptors so that we don\'t have them
      hanging around in clients.  Note that we want to do this after
      initgroups, because at least on Solaris 2.3 it leaves file descriptors
      open. */
   endgrent();
 
+#ifdef HAVE_SIA
+  /* Close later on, after calling sia_become_user().  Closing now could
+     break that call. */
+#else /* HAVE_SIA */
   for (i = 3; i < 1000; i++)
     close(i);
+#endif /* HAVE_SIA */
 
 #ifdef CRAY   /* set up accounting account number, job, limits, permissions  */
   if (cray_setup(ssh_user_uid(uc), ssh_user_name(uc)) < 0)
@@ -744,7 +783,7 @@ Boolean ssh_user_become(SshUser uc)
         }
 #ifdef HAVE_INITGROUPS
       /* Initialize the group list. */
-      if (initgroups((char *)ssh_user_name(uc), ssh_user_gid(uc)) < 0)
+      if (initgroups(ssh_user_name(uc), ssh_user_gid(uc)) < 0)
         {
           SSH_DEBUG(2, ("ssh_user_become: initgroups: %s", strerror(errno)));
           return FALSE;
@@ -752,6 +791,22 @@ Boolean ssh_user_become(SshUser uc)
 #endif /* HAVE_INITGROUPS */
       endgrent();
           
+#ifdef HAVE_SIA
+      /* Temporarily switch to the desired uid.  We'll permanently switch
+         in ssh_user_become_real(). */
+      if (seteuid(ssh_user_uid(uc)) < 0)
+        {
+          SSH_DEBUG(2, ("ssh_user_become: seteuid %d: %s",
+                        (int)ssh_user_uid(uc), strerror(errno)));
+          return FALSE;
+        }
+      if (geteuid() != ssh_user_uid(uc))
+        {
+          SSH_DEBUG(2, ("ssh_user_become: failed to set euid to %d.",
+                        (int)ssh_user_uid(uc)));
+          return FALSE;
+        }
+#else /* HAVE_SIA */
 #ifdef HAVE_SETLUID
   /* Set login uid, if we have setluid(). */
       if (setluid(ssh_user_uid(uc)) < 0)
@@ -774,12 +829,84 @@ Boolean ssh_user_become(SshUser uc)
                         (int)ssh_user_uid(uc)));
           return FALSE;
         }
+#endif /* HAVE_SIA */
     }
   
   /* We are now running with the user's privileges. */
   return TRUE;
 }
 
+#ifdef HAVE_SIA
+/* Last chance to finish anything that ssh_user_become() left undone.  The
+   difference between the two functions is that ssh_user_become() is called
+   before the user's environment is set, while we're called after.
+
+   Switches the current process to the permissions and privileges of the
+   specified user.  The process should not hold any confidential information
+   at this point.  This returns FALSE if switching to the given user failed
+   for some reason.  The return value of this function MUST BE CHECKED! */
+Boolean ssh_user_become_real(SshUser uc,
+                             const char *remote_host,
+                             const char *ttyname)
+{
+  int i;
+  int argc;
+  char **argv;
+
+  SSH_DEBUG(7, ("before going-back-to-uid uid = %d and euid = %d", 
+                (int)getuid(), (int)geteuid()));
+  /* Switch back temporarily to our own uid (typically root, but
+     not always) in case sia_become_user() needs to access protected
+     account databases. */
+  if (seteuid(getuid()) != 0)
+    {
+      SSH_DEBUG(2, ("ssh_user_become_real: seteuid %d: %s",
+                    (int)getuid(), strerror(errno)));
+      return FALSE;
+    }
+
+  SSH_DEBUG(7, ("after seteuid-to-uid uid = %d and euid = %d", 
+                (int)getuid(), (int)geteuid()));
+
+  get_sia_args(&argc, &argv);
+  /* Enhanced Security calls setluid() here in sia_become_user(). */
+  if (sia_become_user(sia_collect_trm, argc, argv, (char *)remote_host,
+                      (char *)ssh_user_name(uc), (char *)ttyname, 0, NULL,
+                      NULL, SIA_BEU_REALLOGIN) != SIASUCCESS)
+    {
+      SSH_DEBUG(2, ("ssh_user_become_real: sia_become_user() failed for user %d.",
+                    (int)ssh_user_uid(uc)));
+      return FALSE;
+    }
+  /* Our sia calls are complete, so we can finally close any extra
+     open file descriptors. */
+  for (i = getdtablesize() - 1; i > 2; i--)
+    close(i);
+
+  /* We can now completely become the user.  We need to do this
+     with setreuid() instead of setuid() because sia_become_user()
+     has set our euid to the user's uid. */
+  if (setreuid(ssh_user_uid(uc), ssh_user_uid(uc)) < 0)
+    {
+      SSH_DEBUG(2, ("ssh_user_become_real: setreuid %d: %s",
+                    (int)ssh_user_uid(uc), strerror(errno)));
+      return FALSE;
+    }
+
+  if (getuid() != ssh_user_uid(uc) || geteuid() != ssh_user_uid(uc))
+    {
+      SSH_DEBUG(2, ("ssh_user_become_real: failed to set uids to %d.",
+                    (int)ssh_user_uid(uc)));
+      return FALSE;
+    }
+
+  SSH_DEBUG(7, ("after completion uid = %d and euid = %d", 
+                (int)getuid(), (int)geteuid()));
+
+  /* We are now running with the user's complete privileges. */
+  return TRUE;
+}
+#endif /* HAVE_SIA */
 
 /* Returns the login name of the user. */
 
@@ -788,21 +915,6 @@ const char *ssh_user_name(SshUser uc)
   return uc->name;
 }
 
-/* Returns the group name of the user. */
-const char *ssh_group_name(SshUser uc)
-{
-  if (uc->group == NULL)
-    {
-      struct group *grp = getgrgid(ssh_user_gid(uc));
-      
-      if (grp != NULL)
-        uc->group = ssh_xstrdup(grp->gr_name);
-      else
-        ssh_dsprintf(&uc->group, "%lu", (unsigned long)(ssh_user_gid(uc)));
-    }
-  return uc->group;
-}
-
 /* Returns the uid of the user.  This is unix-specific. */
 
 uid_t ssh_user_uid(SshUser uc)
