@@ -38,8 +38,15 @@ Boolean ssh_user_free(SshUser uc, Boolean undo);
 Boolean ssh_user_login_is_allowed(SshUser uc);
 
 /* Returns TRUE if login is allowed with the given local password. */
+#ifdef HAVE_SIA
+Boolean ssh_user_validate_local_password(SshUser uc,
+                                         const char *password,
+                                         const char *remote_host);
+#else /* HAVE_SIA */
 Boolean ssh_user_validate_local_password(SshUser uc,
                                          const char *password);
+#endif /* HAVE_SIA */
+
 
 /* Returns TRUE if the user's password needs to be changed. */
 Boolean ssh_user_password_must_be_changed(SshUser uc,
@@ -70,15 +77,25 @@ Boolean ssh_user_validate_secure_rpc_password(SshUser uc,
    The return value of this function MUST BE CHECKED! */
 Boolean ssh_user_become(SshUser uc);
 
+#ifdef HAVE_SIA
+/* Last chance to finish anything that ssh_user_become() left undone.  The
+   difference between the two functions is that ssh_user_become() is called
+   before the user's environment is set, while we're called after.
+
+   Switches the current process to the permissions and privileges of the
+   specified user.  The process should not hold any confidential information
+   at this point.  This returns FALSE if switching to the given user failed
+   for some reason.  The return value of this function MUST BE CHECKED! */
+Boolean ssh_user_become_real(SshUser uc,
+                             const char *remote_host,
+                             const char *ttyname);
+#endif /* HAVE_SIA */
 
 /* Returns the login name of the user. */
 const char *ssh_user_name(SshUser uc);
 
 #ifndef WINDOWS
 
-/* Returns the group name of the user. */
-const char *ssh_group_name(SshUser uc);
-
 /* Returns the uid of the user.  This is unix-specific. */
 uid_t ssh_user_uid(SshUser uc);
 
