@@ -15,6 +15,11 @@ In particular, this updates:
 
 */
 
+/* required to get WTMPX_FILE and updwtmpx() in newer glibc */
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include "sshsessionincludes.h"
 #ifdef HAVE_UTIL_H
 #include <util.h>
@@ -43,16 +48,9 @@ In particular, this updates:
 #endif /* HAVE_ARPA_INET_H */
 #include "sshuser.h"
 
-/* Prototypes for those systems, where they are missing... */
-#ifndef HAVE_PROTO_LOGIN
-void login(const struct utmp *);
-#endif /* !HAVE_PROTO_LOGIN */
-#ifndef HAVE_PROTO_LOGOUT
-int logout(const char *);
-#endif /* !HAVE_PROTO_LOGOUT */
-#ifndef HAVE_PROTO_LOGWTMP
-void logwtmp(const char *, const char *, const char *);
-#endif /* !HAVE_PROTO_LOGWTMP */
+/* Internal function, defined in sshunixtcp.c */
+extern Boolean ssh_string_to_addr(const char *s, struct in_addr *addr);
+
 
 /* Returns the time when the user last logged in, and name of the host
    from which the user logged in from.  Returns 0 if the information
@@ -202,9 +200,9 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
 #if defined(HAVE_LASTLOG_H) || defined(HAVE_LASTLOG)
   struct lastlog ll;
   char *lastlog;
-# ifdef LASTLOG_IS_DIR
+#ifdef LASTLOG_IS_DIR
   char lastlogfile[0x100];
-# endif /* LASTLOG_IS_DIR */
+#endif /* LASTLOG_IS_DIR */
 #endif /* HAVE_LASTLOG_H || HAVE_LASTLOG */
 
 #if defined(HAVE_UTMP_H) && !defined(HAVE_UTMPX_H)
@@ -213,39 +211,39 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
 
   /* Construct an utmp/wtmp entry. */
   memset(&u, 0, sizeof(u));
-# ifdef DEAD_PROCESS
+#ifdef DEAD_PROCESS
   if (user == NULL)
     u.ut_type = DEAD_PROCESS; /* logout */
   else
     u.ut_type = USER_PROCESS;
-# endif /* DEAD_PROCESS */
-# ifdef HAVE_PID_IN_UTMP
+#endif /* LOGIN_PROCESS */
+#ifdef HAVE_PID_IN_UTMP
   u.ut_pid = pid;
-# endif /* PID_IN_UTMP */
-# ifdef HAVE_ID_IN_UTMP
-#  if defined(__sgi) || defined(CRAY)
+#endif /* PID_IN_UTMP */
+#ifdef HAVE_ID_IN_UTMP
+#if defined(__sgi) || defined(CRAY)
     strncpy(u.ut_id, ttyname + 8, sizeof(u.ut_id)); /* /dev/ttyq99 -> q99 */
-#  else /* __sgi */
+#else /* __sgi */
     if (sizeof(u.ut_id) > 4)
       strncpy(u.ut_id, ttyname + 5, sizeof(u.ut_id));
     else
       strncpy(u.ut_id, ttyname + strlen(ttyname) - 2, sizeof(u.ut_id));
-#  endif /* __sgi */
-# endif /* HAVE_ID_IN_UTMP */
+#endif /* __sgi */
+#endif /* HAVE_ID_IN_UTMP */
   strncpy(u.ut_line, ttyname + 5, sizeof(u.ut_line));
   u.ut_time = ssh_time();
-# ifdef HAVE_NAME_IN_UTMP
+#ifdef HAVE_NAME_IN_UTMP
   strncpy(u.ut_name, user ? ssh_user_name(user) : "", sizeof(u.ut_name));
-# else /* HAVE_NAME_IN_UTMP */
+#else /* HAVE_NAME_IN_UTMP */
   strncpy(u.ut_user, user ? ssh_user_name(user) : "", sizeof(u.ut_user));
-# endif /* HAVE_NAME_IN_UTMP */
-# ifdef HAVE_HOST_IN_UTMP
+#endif /* HAVE_NAME_IN_UTMP */
+#ifdef HAVE_HOST_IN_UTMP
   strncpy(u.ut_host, host, sizeof(u.ut_host));
   if (strlen(host) > sizeof(u.ut_host)) {
     strncpy(u.ut_host, ip, sizeof(u.ut_host));
   }
-# endif /* HAVE_HOST_IN_UTMP */
-# ifdef HAVE_ADDR_IN_UTMP
+#endif /* HAVE_HOST_IN_UTMP */
+#ifdef HAVE_ADDR_IN_UTMP
   if (ip && *ip)
     {
       struct in_addr sin_addr;
@@ -255,24 +253,24 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
     }
   else
     memset(&u.ut_addr, 0, sizeof(u.ut_addr));
-# endif
+#endif
   /* Figure out the file names. */
-# ifdef _PATH_UTMP
+#ifdef _PATH_UTMP
   utmp = _PATH_UTMP;
   wtmp = _PATH_WTMP;
-# else
-#  ifdef UTMP_FILE
+#else
+#ifdef UTMP_FILE
   utmp = UTMP_FILE;
   wtmp = WTMP_FILE;
-#  else
+#else
   utmp = SSH_UTMP;
   wtmp = SSH_WTMP;
-#  endif
-# endif
+#endif
+#endif
   
-# ifdef HAVE_LIBUTIL_LOGIN
+#ifdef HAVE_LIBUTIL_LOGIN
   login(&u);
-# else /* HAVE_LIBUTIL_LOGIN */
+#else /* HAVE_LIBUTIL_LOGIN */
   /* Append an entry to wtmp. */
   fd = open(wtmp, O_WRONLY|O_APPEND);
   if (fd >= 0)
@@ -287,9 +285,9 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
   fd = open(utmp, O_RDWR);
   if (fd >= 0)
     {
-#  ifdef HAVE_TTYSLOT
+#ifdef HAVE_TTYSLOT
       int n = ttyslot();
-#   if defined(ultrix) || defined(NeXT)
+#if defined(ultrix) || defined(NeXT)
       /* the problem is that Berkeley unix uses ttyslot() to determine
        * where in the utmp file to write and it is correct at login
        * time because the controlling tty is correct.  At logout time,
@@ -297,15 +295,15 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
        * different (or no) controlling tty so we must search for the
        * right record to clobber.  -- corey 5/7/97 */
       if (n > 0 && user != NULL) {
-#   else /* ultrix || NeXT */
+#else /* ultrix || NeXT */
       if (n > 0) {
-#   endif /* ultrix || NeXT */
+#endif /* ultrix || NeXT */
         lseek(fd, (off_t)(n*sizeof(u)), 0);
         if (write(fd, &u, sizeof(u)) != sizeof(u))
           ssh_warning("Could not write to %.100s: %.100s", 
                       utmp, strerror(errno));
       } else
-#  endif /* HAVE_TTYSLOT */
+#endif /* HAVE_TTYSLOT */
       while (1)
         {
           off_t offset;
@@ -319,11 +317,11 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
                             utmp, strerror(errno));
               break;
             }
-#  if defined(ultrix) || defined(NeXT)            /* corey */
+#if defined(ultrix) || defined(NeXT)            /* corey */
           if (strcmp(u2.ut_line, ttyname + 5) == 0 && *u2.ut_name)
-#  else   /* ultrix || NeXT */
+#else   /* ultrix || NeXT */
           if (strncmp(u2.ut_line, ttyname + 5, sizeof(u2.ut_line)) == 0)
-#  endif  /* ultrix || NeXT */
+#endif  /* ultrix || NeXT */
             {
               lseek(fd, offset, 0);
               if (write(fd, &u, sizeof(u)) != sizeof(u))
@@ -334,7 +332,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
         }
       close(fd);
     }
-# endif /* HAVE_LIBUTIL_LOGIN */
+#endif /* HAVE_LIBUTIL_LOGIN */
 #endif /* HAVE_UTMP_H && !HAVE_UTMPX_H */
 
 #ifdef HAVE_UTMPX_H
@@ -342,9 +340,9 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
     struct utmpx ux, *uxp;
     memset(&ux, 0, sizeof(ux));
     strncpy(ux.ut_line, ttyname + 5, sizeof(ux.ut_line));
+    setutxent(); /* open the database and reset to first position */
     if (user == NULL)
       {
-        setutxent(); /* open the database and reset to first position */
         /* logout; find previous entry for pid and zonk it */
         while ((uxp = getutxent()))
           {
@@ -353,7 +351,6 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
             ux = *uxp;
             break;
           }
-        endutxent();
       }
     else
       {
@@ -364,13 +361,10 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
         strncpy(ux.ut_user, ssh_user_name(user), sizeof(ux.ut_user));
         ux.ut_type = USER_PROCESS;
       }
-# ifdef DEAD_PROCESS
-    if (user == NULL)
-      ux.ut_type = DEAD_PROCESS; /* logout */
-# endif /* DEAD_PROCESS */
-# if defined(__sgi) || defined(SCO) || defined(linux)
+    endutxent();
+#if defined(__sgi) || defined(SCO) || defined(linux)
     strncpy(ux.ut_id, ttyname + 8, sizeof(ux.ut_id)); /* /dev/ttyq99 -> q99 */
-# else /* __sgi || SCO || linux */
+#else /* __sgi || SCO || linux */
     if (sizeof(ux.ut_id) > 4)
       { 
         strncpy(ux.ut_id, ttyname + 5, sizeof(ux.ut_id));
@@ -378,7 +372,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
     else
       {
         char buf[20];
-#  ifdef HAVE_MINOR
+#ifdef HAVE_MINOR
         struct stat st;
         
         buf[0] = 0;
@@ -387,7 +381,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
           snprintf(buf, sizeof (buf), "P%03d", (int)minor(st.st_rdev));
         }
         strncpy(ux.ut_id, buf, sizeof(ux.ut_id));
-#  else /* HAVE_MINOR */
+#else /* HAVE_MINOR */
         /* if we don't have minor, we just dig out the last <= three letters
            from ttyname. */
         
@@ -400,33 +394,30 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
           {
             snprintf(buf, sizeof (buf), "P%s", ttyname);
           }
-#  endif /* HAVE_MINOR */
+#endif /* HAVE_MINOR */
       }
-# endif /* __sgi || SCO || linux */
-# ifdef HAVE_GETTIMEOFDAY
-#  ifdef HAVE_NO_TZ_IN_GETTIMEOFDAY
+#endif /* __sgi || SCO || linux */
+    ux.ut_pid = pid;
+
+#ifdef HAVE_GETTIMEOFDAY
+#ifdef HAVE_NO_TZ_IN_GETTIMEOFDAY
     gettimeofday(&ux.ut_tv);
-#  else
+#else
     gettimeofday(&ux.ut_tv, NULL);
-#  endif
-# else /* HAVE_GETTIMEOFDAY */
+#endif
+#else /* HAVE_GETTIMEOFDAY */
     ux.ut_tv.tv_sec = ssh_time();
     ux.ut_tv.tv_usec = 0;
-# endif /* HAVE_GETTIMEOFDAY */
+#endif /* HAVE_GETTIMEOFDAY */
 
-# ifdef HAVE_PID_IN_UTMPX
-    ux.ut_pid = pid;
-# else /* HAVE_PID_IN_UTMPX */
-#  ifdef HAVE_SESSION_IN_UTMPX
     ux.ut_session = pid;
-#  endif /* HAVE_SESSION_IN_UTMPX */
-# endif /* HAVE_PID_IN_UTMPX */
     strncpy(ux.ut_host, host, sizeof(ux.ut_host));
-    ux.ut_host[sizeof(ux.ut_host) - 1] = '\0';
-# ifdef HAVE_SYSLEN_IN_UTMPX
+    ux.ut_host[sizeof(ux.ut_host) - 1] = 0;
+#ifdef HAVE_SYSLEN_IN_UTMPX
     ux.ut_syslen = strlen(ux.ut_host);
-# endif /* HAVE_SYSLEN_IN_UTMPX */
-# ifdef HAVE_MAKEUTX
+#endif /* HAVE_SYSLEN_IN_UTMPX */
+    setutxent(); /* reopen database and reset position to first */
+#ifdef HAVE_MAKEUTX
     /*
      * modutx/makeutx notify init(1) to clean up utmpx for this pid
      * automatically if we don't manage to, for some odd reason
@@ -435,27 +426,25 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
         modutx(&ux);
     else
         makeutx(&ux);
-# else
+#else
     pututxline(&ux);
-# if defined(HAVE_UPDWTMPX) && defined(WTMPX_FILE)
     updwtmpx(WTMPX_FILE, &ux);
-# endif /* HAVE_UPDWTMPX && WTMPX_FILE*/
-# endif /* HAVE_MAKEUTX */
+#endif
     endutxent();
   }
 #endif /* HAVE_UTMPX_H */
 
 #if defined(HAVE_LASTLOG_H) || defined(HAVE_LASTLOG)
 
-# ifdef _PATH_LASTLOG
+#ifdef _PATH_LASTLOG
   lastlog = _PATH_LASTLOG;
-# else
-#  ifdef LASTLOG_FILE
+#else
+#ifdef LASTLOG_FILE
   lastlog = LASTLOG_FILE;
-#  else
+#else
   lastlog = SSH_LASTLOG;
-#  endif
-# endif
+#endif
+#endif
 
   /* Update lastlog unless actually recording a logout. */
   if (user != NULL) /* only on login ... */
@@ -468,7 +457,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
       ll.ll_time = ssh_time();
       strncpy(ll.ll_line, ttyname + 5, sizeof(ll.ll_line));
       strncpy(ll.ll_host, host, sizeof(ll.ll_host));
-# ifdef LASTLOG_IS_DIR
+#ifdef LASTLOG_IS_DIR
       snprintf(lastlogfile, 
                sizeof (lastlogfile), 
                "%.100s/%.100s", 
@@ -487,7 +476,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
           ssh_warning("Could not open %.100s: %.100s",
                       lastlogfile, strerror(errno));
         }
-# else /* LASTLOG_IS_DIR */
+#else /* LASTLOG_IS_DIR */
       fd = open(lastlog, O_RDWR);
       if (fd >= 0)
         {
@@ -497,7 +486,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
                         lastlog, strerror(errno));
           close(fd);
         }
-# endif /* LASTLOG_IS_DIR */
+#endif /* LASTLOG_IS_DIR */
     }
 #endif /* HAVE_LASTLOG_H || HAVE_LASTLOG */
 
@@ -508,9 +497,6 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
       int lasttime = ssh_time();
       if (setuserdb(S_WRITE) < 0)
         ssh_warning("setuserdb S_WRITE failed: %.100s", strerror(errno));
-      /* XXX putuserattr(char *, char *, void *, int) I don't think
-       that casting an integer to pointer to void is the correct way
-       here. */
       if (putuserattr((char *)ssh_user_name(user),
                       S_LASTTIME, (void *)lasttime, SEC_INT) < 0)
         ssh_warning("putuserattr S_LASTTIME failed: %.100s", strerror(errno));
