@@ -8,7 +8,7 @@ Copyright (c) 1996 SSH Communications Security, Finland
                    All rights reserved
 
 Created: Thu Oct 24 20:38:23 1996 ylo
-Last modified: 18:05 May 28 2000 kivinen
+Last modified: 04:10 May 24 1998 kivinen
 
 */
 
@@ -20,12 +20,12 @@ int main()
 {
   int pass, subpass, i;
   unsigned char data[1024];
-  SshBufferStruct b;
-  SshBuffer bufferp;
+  SshBuffer b, *bufferp;
   char *s, *s2;
   unsigned char *cp;
   unsigned int len, origlen;
   size_t len2;
+  long value;
   SshInt mp1, mp2;
 
   /* test strings for some cases of ssh2 strings */
@@ -120,6 +120,7 @@ int main()
       
       for (i = 0; i < 1000; i++)
         {
+          buffer_put_vlint32(&b, i * 1234567L);
           buffer_put_char(&b, i);
           buffer_put_uint32_string(&b, s, len);
         }
@@ -140,6 +141,13 @@ int main()
         }
       for (i = 0; i < 1000; i++)
         {
+          value = buffer_get_vlint32(&b);
+          if (value != i * 1234567L)
+            {
+              printf("buffer_get_vlint32 failed: i=%d got %ld exp %ld\n",
+                     i, value, i * 1234567L);
+              exit(1);
+            }
           if (buffer_get_char(&b) != i % 256)
             {
               printf("buffer_get_char failed\n");
@@ -183,9 +191,7 @@ int main()
       if (memcmp(b.buf+b.offset, ssh2test_b[i], ssh2test_l[i]) != 0)
         ssh_fatal("ssh_put_mp_int_ssh2style failed (%d, match)", i);
 
-      if (!buffer_get_mp_int_ssh2style(&b, &mp2))
-        ssh_fatal("ssh_get_mp_int_ssh2style returned FALSE (%d)", i);
-      
+      buffer_get_mp_int_ssh2style(&b, &mp2);
       if (ssh_mp_cmp(&mp1, &mp2) != 0)
         ssh_fatal("ssh_get_mp_int_ssh2style failed (%d)", i);
 
