@@ -18,9 +18,7 @@ Tests for the ssh_encode/decode functions.
 #include "sshincludes.h"
 #include "sshencode.h"
 
-#define SSH_DEBUG_MODULE "Main"
-
-SshBuffer buffer;
+SshBuffer *buffer;
 
 /* This encodes data into a buffer using the given format, and compares the
    result against the expected value.  This also checks that data at the
@@ -40,7 +38,7 @@ void encode_case(const char *name, const char *expect,
     ssh_buffer_append(buffer, &ch, 1);
 
   va_start(va, expect_len);
-  bytes = ssh_encode_buffer_va(buffer, va);
+  bytes = ssh_encode_va(buffer, va);
   if (bytes != expect_len || ssh_buffer_len(buffer) != len + expect_len)
     ssh_fatal("test_encode: %s: unexpected length %d vs. %d",
               name, bytes, len + expect_len);
@@ -75,7 +73,7 @@ void decode_case_str(SshEncodingFormat fmt, const char *value, size_t valuelen)
 
 void decode_case_int(SshEncodingFormat fmt, unsigned int value)
 {
-  SshUInt32 lv;
+  unsigned long lv;
   size_t bytes;
 
   bytes = ssh_buffer_len(buffer);
@@ -129,7 +127,7 @@ void decode_case_data(SshEncodingFormat fmt, const char *value, size_t valuelen)
   char buf[1024];
   size_t bytes;
 
-  SSH_ASSERT(valuelen < sizeof(buf));
+  assert(valuelen < sizeof(buf));
   bytes = ssh_buffer_len(buffer);
   if (bytes != ssh_decode_array(ssh_buffer_ptr(buffer), ssh_buffer_len(buffer),
                                       fmt, NULL, valuelen, SSH_FORMAT_END))
@@ -145,6 +143,21 @@ void decode_case_data(SshEncodingFormat fmt, const char *value, size_t valuelen)
 
 void test_encode(void)
 {
+  encode_case("vlint32_str empty", "\0", 1,
+              SSH_FORMAT_VLINT32_STR, "", 0, SSH_FORMAT_END);
+  decode_case_str(SSH_FORMAT_VLINT32_STR, "", 0);
+  encode_case("vlint32_str empty", "\0", 1,
+              SSH_FORMAT_VLINT32_STR, NULL, 0, SSH_FORMAT_END);
+  decode_case_str(SSH_FORMAT_VLINT32_STR, "", 0);
+  encode_case("vlint32_str 1", "\1A", 2,
+              SSH_FORMAT_VLINT32_STR, "ABC", 1, SSH_FORMAT_END);
+  decode_case_str(SSH_FORMAT_VLINT32_STR, "A", 1);
+  encode_case("vlint32_str null", "\7foo\0bar", 8,
+              SSH_FORMAT_VLINT32_STR, "foo\0bar", 7, SSH_FORMAT_END);
+  decode_case_str(SSH_FORMAT_VLINT32_STR, "foo\0bar", 7);
+  encode_case("vlint32_str 65", "\100\101XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX", 67,
+              SSH_FORMAT_VLINT32_STR, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXABC", 65, SSH_FORMAT_END);
+  decode_case_str(SSH_FORMAT_VLINT32_STR, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXABC", 65);
   encode_case("uint32_str 0", "\0\0\0\0", 4,
               SSH_FORMAT_UINT32_STR, NULL, 0, SSH_FORMAT_END);
   decode_case_str(SSH_FORMAT_UINT32_STR, "", 0);
@@ -154,6 +167,36 @@ void test_encode(void)
   encode_case("uint32_str 5", "\0\0\0\5ABCDE", 9,
               SSH_FORMAT_UINT32_STR, "ABCDEFGHIJK", 5, SSH_FORMAT_END);
   decode_case_str(SSH_FORMAT_UINT32_STR, "ABCDE", 5);
+  encode_case("vlint32 0", "\0", 1,
+              SSH_FORMAT_VLINT32, (SshUInt32) 0, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 0);
+  encode_case("vlint32 63", "\77", 1,
+              SSH_FORMAT_VLINT32, (SshUInt32) 63, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 63);
+  encode_case("vlint32 64", "\100\100", 2,
+              SSH_FORMAT_VLINT32, (SshUInt32) 64, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 64);
+  encode_case("vlint32 255", "\100\377", 2,
+              SSH_FORMAT_VLINT32, (SshUInt32) 255, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 255);
+  encode_case("vlint32 256", "\101\0", 2,
+              SSH_FORMAT_VLINT32, (SshUInt32) 256, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 256);
+  encode_case("vlint32 16383", "\177\377", 2,
+              SSH_FORMAT_VLINT32, (SshUInt32) 16383, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 16383);
+  encode_case("vlint32 16384", "\200\100\0", 3,
+              SSH_FORMAT_VLINT32, (SshUInt32) 16384, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 16384);
+  encode_case("vlint32 4194303", "\277\377\377", 3,
+              SSH_FORMAT_VLINT32, (SshUInt32) 4194303, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 4194303);
+  encode_case("vlint32 4194304", "\300\0\100\0\0", 5,
+              SSH_FORMAT_VLINT32, (SshUInt32) 4194304, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 4194304);
+  encode_case("vlint32 2^32-1", "\300\377\377\377\377", 5,
+              SSH_FORMAT_VLINT32, (SshUInt32) 0xffffffff, SSH_FORMAT_END);
+  decode_case_int(SSH_FORMAT_VLINT32, (SshUInt32) 0xffffffff);
   encode_case("uint32 0x12345678", "\22\64\126\170", 4,
               SSH_FORMAT_UINT32, (SshUInt32) 0x12345678, SSH_FORMAT_END);
   decode_case_int(SSH_FORMAT_UINT32, (SshUInt32) 0x12345678);
@@ -183,10 +226,18 @@ void test_encode(void)
 
 void test_empty_decode(void)
 {
+  if (ssh_decode_array((const unsigned char *) "", 0,
+                       SSH_FORMAT_VLINT32_STR, NULL, NULL,
+                       SSH_FORMAT_END) != 0)
+    ssh_fatal("test_empty_decode failed");
   if (ssh_decode_array((const unsigned char *) "", 0,
                        SSH_FORMAT_UINT32_STR, NULL, NULL,
                        SSH_FORMAT_END) != 0)
     ssh_fatal("test_empty_decode failed");
+  if (ssh_decode_array((const unsigned char *) "", 0,
+                       SSH_FORMAT_VLINT32, NULL,
+                       SSH_FORMAT_END) != 0)
+    ssh_fatal("test_empty_decode failed");
   if (ssh_decode_array((const unsigned char *) "", 0,
                        SSH_FORMAT_UINT32, NULL, 
                        SSH_FORMAT_END) != 0)
@@ -217,8 +268,12 @@ void test_random_decode(void)
   for (i = 0; i < sizeof(buf); i++)
     buf[i] = rand();
   
+  ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_VLINT32_STR, NULL, NULL,
+                         SSH_FORMAT_END);
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_UINT32_STR, NULL, NULL,
                          SSH_FORMAT_END);
+  ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_VLINT32, NULL,
+                         SSH_FORMAT_END);
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_UINT32, NULL,
                          SSH_FORMAT_END);
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_CHAR, NULL,
@@ -231,7 +286,7 @@ void test_random_decode(void)
                          SSH_FORMAT_END);
 } 
 
-void test_functions_parse_compound(SshBuffer buffer, ...)
+void test_functions_parse_compound(SshBuffer *buffer, ...)
 {
   va_list va;
 
@@ -247,23 +302,60 @@ void test_functions(int foo, ...)
   unsigned char *cp;
   
   ssh_buffer_clear(buffer);
+  if (ssh_encode_buffer(buffer, SSH_FORMAT_VLINT32, (SshUInt32) 67,
+                        SSH_FORMAT_END) != 2)
+    ssh_fatal("test_functions: ssh_encode error");
+  if (memcmp(ssh_buffer_ptr(buffer), "\100\103", 2) != 0)
+    ssh_fatal("test_functions: ssh_encode data error");
 
   ssh_buffer_clear(buffer);
   va_start(va, foo);
-  if (ssh_encode_buffer_va(buffer, va) != 2)
-    ssh_fatal("test_functions: ssh_encode_buffer_va error");
+  if (ssh_encode_va(buffer, va) != 2)
+    ssh_fatal("test_functions: ssh_encode_va error");
   if (memcmp(ssh_buffer_ptr(buffer), "\100\103", 2) != 0)
-    ssh_fatal("test_functions: ssh_encode_buffer_va data error");
+    ssh_fatal("test_functions: ssh_encode_va data error");
 
-
-  if (ssh_encode_array_alloc_va(NULL, va) != 2)
-    ssh_fatal("test_function: ssh_encode_array_alloc_va NULL error");
-  if (ssh_encode_array_alloc_va(&cp, va) != 2)
-    ssh_fatal("test_functions: ssh_encode_array_alloc_va error");
+  if (ssh_encode_alloc(NULL, SSH_FORMAT_VLINT32, (SshUInt32) 67,
+                       SSH_FORMAT_END) != 2)
+    ssh_fatal("test_function: ssh_encode_alloc NULL error");
+  if (ssh_encode_alloc(&cp, SSH_FORMAT_VLINT32, (SshUInt32) 67,
+                       SSH_FORMAT_END) != 2)
+    ssh_fatal("test_functions: ssh_encode_alloc error");
   if (memcmp(cp, "\100\103", 2) != 0)
-    ssh_fatal("test_functions: ssh_encode_array_alloc_va data error");
+    ssh_fatal("test_functions: ssh_encode_alloc data error");
+  ssh_xfree(cp);
+  cp = NULL;
+
+  if (ssh_encode_alloc_va(NULL, va) != 2)
+    ssh_fatal("test_function: ssh_encode_alloc_va NULL error");
+  if (ssh_encode_alloc_va(&cp, va) != 2)
+    ssh_fatal("test_functions: ssh_encode_alloc_va error");
+  if (memcmp(cp, "\100\103", 2) != 0)
+    ssh_fatal("test_functions: ssh_encode_alloc_va data error");
   ssh_xfree(cp);
 
+  /* Compound test. */
+  ssh_buffer_clear(buffer);
+  if (ssh_encode_buffer(buffer,
+                        SSH_FORMAT_VLINT32_STR, "A", 1,
+                        SSH_FORMAT_BOOLEAN, FALSE,
+                        SSH_FORMAT_VLINT32, (SshUInt32) 7,
+                        SSH_FORMAT_END) != 4)
+    ssh_fatal("test_functions: compound error");
+  if (memcmp(ssh_buffer_ptr(buffer), "\1A\0\7", 4) != 0)
+    ssh_fatal("test_functions: compound data error");
+
+  {
+    Boolean bool;
+    SshUInt32 l = 0;
+    test_functions_parse_compound(buffer,
+                                  SSH_FORMAT_VLINT32_STR, NULL, NULL,
+                                  SSH_FORMAT_BOOLEAN, &bool,
+                                  SSH_FORMAT_VLINT32, &l,
+                                  SSH_FORMAT_END);
+    if (bool != FALSE || l != 7)
+      ssh_fatal("test_functions: compound parse error");
+  }
 }
 
 int main()
@@ -276,9 +368,11 @@ int main()
       test_encode();
       test_empty_decode();
       test_random_decode();
+      test_functions(0, SSH_FORMAT_VLINT32, (SshUInt32) 67, SSH_FORMAT_END);
       ssh_buffer_free(buffer);
     }
 
   return 0;
 }
 
+/* XXX add SSH_FORMAT_EXTENDED tests. */
